﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Data;
using System.Drawing;
using System.Web.UI.HtmlControls;
using System.IO;

using Image = System.Web.UI.WebControls.Image;

using CMS.UIControls;
using CMS.Controls;
using CMS.GlobalHelper;
using CMS.ExtendedControls;
using CMS.TreeEngine;
using CMS.WorkflowEngine;
using CMS.CMSHelper;
using CMS.SettingsProvider;
using CMS.SiteProvider;

using TreeNode = CMS.TreeEngine.TreeNode;


public partial class CMSAdminControls_Dialogs_LinkMediaSelector_InnerMediaView : CMSUserControl
{
    #region "Private variables"

    private const double MaxThumbImgWidth = 160.0;
    private const double MaxThumbImgHeight = 95.0;
    private const int PAGER_GROUP_SIZE = 10;

    private OutputFormatEnum mOutputFormat = OutputFormatEnum.HTMLMedia;
    private DialogViewModeEnum mViewMode = DialogViewModeEnum.ListView;
    private DataSet mDataSource = null;
    private SelectableContentEnum mSelectableContent = SelectableContentEnum.AllContent;
    private MediaSourceEnum mSourceType = MediaSourceEnum.Content;
    private TreeNode mTreeNodeObj = null;

    private int mVersionHistoryId = 0;

    private string mFileIdColumn = "";
    private string mFileNameColumn = "";
    private string mFileExtensionColumn = "";
    private string mFileSizeColumn = "";
    private string mFileWidthColumn = "";
    private string mFileHeightColumn = "";

    private string mAllowedExtensions = "";
    private int mResizeToHeight = 0;
    private int mResizeToWidth = 0;
    private int mResizeToMaxSideSize = 0;

    private string mInfoText = "";
    private int idCount = 0;
    private int mNodeParentNodeId = 0;

    private int mLastSiteId = 0;

    #endregion


    #region "Events & delegates"

    /// <summary>
    /// Delegate for an event occuring when argument set is required.
    /// </summary>
    /// <param name="dr">DataRow holding information on currently processed file.</param>    
    public delegate string OnGetArgumentSet(DataRow dr);


    /// <summary>
    /// Event occuring when argument set is required.
    /// </summary>
    public event OnGetArgumentSet GetArgumentSet;


    /// <summary>
    /// Delegate for the event fired when URL for list image is required.
    /// </summary>
    /// <param name="dr">DataRow holding information on currently processed file.</param>   
    /// <param name="isPreview">Indicates whether the image is generated as part of preview.</param>
    /// <param name="notAttachment">Indicates whether the URL is required for non-attachment item.</param>
    public delegate string OnGetListItemUrl(DataRow dr, bool isPreview, bool notAttachment);


    /// <summary>
    /// Event occuring when URL for list item image is required.
    /// </summary>
    public event OnGetListItemUrl GetListItemUrl;


    /// <summary>
    /// Delegate for the event fired when URL for tiles & thumbnails image is required.
    /// </summary>
    /// <param name="dr">DataRow holding information on currently processed file.</param>  
    /// <param name="isPreview">Indicates whether the image is generated as part of preview.</param>
    /// <param name="maxSideSize">Maximum size of the preview image. If full-size required parameter gets zero value.</param>
    /// <param name="notAttachment">Indicates whether the URL is required for non-attachment item.</param>
    public delegate string OnGetTilesThumbsItemUrl(DataRow dr, bool isPreview, int height, int width, int maxSideSize, bool notAttachment);


    /// <summary>
    /// Event occuring when URL for tiles & thumbnails image is required.
    /// </summary>
    public event OnGetTilesThumbsItemUrl GetTilesThumbsItemUrl;

    #endregion


    #region "Public properties"


    /// <summary>
    /// Gets or sets the OutputFormat (needed for correct dialog type reckognition)
    /// </summary>
    public OutputFormatEnum OutputFormat
    {
        get
        {
            return this.mOutputFormat;
        }
        set
        {
            this.mOutputFormat = value;
        }
    }


    /// <summary>
    /// Gets a UniGrid control used to display files in LIST view mode.
    /// </summary>
    public UniGrid ListViewControl
    {
        get
        {
            return this.gridList;
        }
    }


    /// <summary>
    /// Gets a repeater control used to display files in TILES view mode.
    /// </summary>
    public BasicRepeater TilesViewControl
    {
        get
        {
            return this.repTilesView;
        }
    }


    /// <summary>
    /// Gets a repeater control used to display files in THUMBNAILS view mode.
    /// </summary>
    public BasicRepeater ThumbnailsViewControl
    {
        get
        {
            return this.repThumbnailsView;
        }
    }


    /// <summary>
    /// Gets or sets a view mode used to display files.
    /// </summary>
    public DialogViewModeEnum ViewMode
    {
        get
        {
            return this.mViewMode;
        }
        set
        {
            this.mViewMode = value;
        }
    }


    /// <summary>
    /// Gets or sets source of the data for view controls.
    /// </summary>
    public DataSet DataSource
    {
        get
        {
            return this.mDataSource;
        }
        set
        {
            this.mDataSource = value;
        }
    }


    /// <summary>
    /// Type of the content which can be selected.
    /// </summary>
    public SelectableContentEnum SelectableContent
    {
        get
        {
            return this.mSelectableContent;
        }
        set
        {
            this.mSelectableContent = value;
        }
    }


    /// <summary>
    /// Gets or sets name of the column holding information on the file identifier.
    /// </summary>
    public string FileIdColumn
    {
        get
        {
            return this.mFileIdColumn;
        }
        set
        {
            this.mFileIdColumn = value;
        }
    }

    /// <summary>
    /// Gets or sets name of the column holding information on file name.
    /// </summary>
    public string FileNameColumn
    {
        get
        {
            return this.mFileNameColumn;
        }
        set
        {
            this.mFileNameColumn = value;
        }
    }


    /// <summary>
    /// Gets or sets name of the column holding information on file extension.
    /// </summary>
    public string FileExtensionColumn
    {
        get
        {
            return this.mFileExtensionColumn;
        }
        set
        {
            this.mFileExtensionColumn = value;
        }
    }


    /// <summary>
    /// Gets or sets name of the column holding information on file width.
    /// </summary>
    public string FileWidthColumn
    {
        get
        {
            return this.mFileWidthColumn;
        }
        set
        {
            this.mFileWidthColumn = value;
        }
    }


    /// <summary>
    /// Gets or sets name of the column holding information on file height.
    /// </summary>
    public string FileHeightColumn
    {
        get
        {
            return this.mFileHeightColumn;
        }
        set
        {
            this.mFileHeightColumn = value;
        }
    }


    /// <summary>
    /// Gets or sets name of the column holding information on file size.
    /// </summary>
    public string FileSizeColumn
    {
        get
        {
            return this.mFileSizeColumn;
        }
        set
        {
            this.mFileSizeColumn = value;
        }
    }


    /// <summary>
    /// Gets or sets text of the information label.
    /// </summary>
    public string InfoText
    {
        get
        {
            return this.mInfoText;
        }
        set
        {
            this.mInfoText = value;
        }
    }


    /// <summary>
    /// Gets the node attachments are related to.
    /// </summary>
    public TreeNode TreeNodeObj
    {
        get
        {
            return mTreeNodeObj;
        }
        set
        {
            mTreeNodeObj = value;
        }
    }


    /// <summary>
    /// Gets or sets information on source type.
    /// </summary>
    public MediaSourceEnum SourceType
    {
        get
        {
            return this.mSourceType;
        }
        set
        {
            this.mSourceType = value;
        }
    }


    /// <summary>
    /// Height of attachment.
    /// </summary>
    public int ResizeToHeight
    {
        get
        {
            return this.mResizeToHeight;
        }
        set
        {
            this.mResizeToHeight = value;
        }
    }


    /// <summary>
    /// Width of attachment.
    /// </summary>
    public int ResizeToWidth
    {
        get
        {
            return this.mResizeToWidth;
        }
        set
        {
            this.mResizeToWidth = value;
        }
    }


    /// <summary>
    /// Max side size of attachment.
    /// </summary>
    public int ResizeToMaxSideSize
    {
        get
        {
            return this.mResizeToMaxSideSize;
        }
        set
        {
            this.mResizeToMaxSideSize = value;
        }
    }


    /// <summary>
    /// Currently topN value for selection.
    /// </summary>
    public int CurrentTopN
    {
        get
        {
            int pageSize = 0;
            int pageIndex = 0;
            int pageCount = 0;
            switch (this.ViewMode)
            {
                case DialogViewModeEnum.TilesView:
                    if (this.pageSizeTiles.SelectedValue.ToUpper() == "##ALL##")
                    {
                        pageSize = 0;
                    }
                    else
                    {
                        pageSize = ValidationHelper.GetInteger(this.pageSizeTiles.SelectedValue, 20);
                    }
                    pageIndex = this.pagerElemTiles.GetCurrentPage();
                    pageCount = this.pagerElemTiles.PageCount;
                    break;

                case DialogViewModeEnum.ThumbnailsView:
                    if (this.pageSizeThumbs.SelectedValue.ToUpper() == "##ALL##")
                    {
                        pageSize = 0;
                    }
                    else
                    {
                        pageSize = ValidationHelper.GetInteger(this.pageSizeThumbs.SelectedValue, 12);
                    }
                    pageIndex = this.pagerElemThumbnails.GetCurrentPage();
                    pageCount = this.pagerElemThumbnails.PageCount;
                    break;

                case DialogViewModeEnum.ListView:
                    if (this.gridList.PageSizeDropdown.SelectedValue.ToUpper() == "##ALL##")
                    {
                        pageSize = 0;
                    }
                    else
                    {
                        pageSize = ValidationHelper.GetInteger(this.gridList.PageSizeDropdown.SelectedValue, 10);
                    }
                    pageIndex = gridList.GridView.PageIndex;
                    pageCount = gridList.GridView.PagerSettings.PageButtonCount;
                    break;
            }
            if (pageSize > 0)
            {
                pageCount = (pageCount > PAGER_GROUP_SIZE) ? pageCount : PAGER_GROUP_SIZE;

                return (pageSize * (pageIndex + 1 + pageCount + (PAGER_GROUP_SIZE - (pageCount % PAGER_GROUP_SIZE))));
            }
            else
            {
                return 0;
            }
        }
    }


    /// <summary>
    /// Indicates whether the content tree is displaying more than max tree nodes.
    /// </summary>
    public bool IsDisplayMore
    {
        get
        {
            return ValidationHelper.GetBoolean(ViewState["IsDisplayMore"], false);
        }
        set
        {
            this.ViewState["IsDisplayMore"] = value;
        }
    }


    /// <summary>
    /// ID of the related node version history.
    /// </summary>
    private int VersionHistoryID
    {
        get
        {
            return this.mVersionHistoryId;
        }
        set
        {
            this.mVersionHistoryId = value;
        }
    }

    #endregion


    #region "Attachment"

    /// <summary>
    /// Gets all allowed extensions.
    /// </summary>
    public string AllowedExtensions
    {
        get
        {
            if (this.mAllowedExtensions == "")
            {
                this.mAllowedExtensions = QueryHelper.GetString("allowedextensions", "");
                if (this.mAllowedExtensions == "")
                {
                    this.mAllowedExtensions = SettingsKeyProvider.GetStringValue(CMSContext.CurrentSiteName + ".CMSUploadExtensions");
                }
            }
            return this.mAllowedExtensions;
        }
    }


    /// <summary>
    /// Gets or sets ID of the parent node.
    /// </summary>
    public int NodeParentID
    {
        get
        {
            return this.mNodeParentNodeId;
        }
        set
        {
            this.mNodeParentNodeId = value;
        }
    }

    #endregion


    protected override void OnPreRender(EventArgs e)
    {
        // Display information on empty data
        bool isEmpty = DataHelper.DataSourceIsEmpty(this.DataSource);
        if (isEmpty)
        {
            this.plcViewArea.Visible = false;
        }
        else
        {
            this.lblInfo.Visible = false;
            this.plcViewArea.Visible = true;
        }

        // If info text is set display it
        if (!string.IsNullOrEmpty(this.InfoText))
        {
            this.lblInfo.Text = this.InfoText;
            this.lblInfo.Visible = true;
        }
        else if (isEmpty)
        {
            this.lblInfo.Text = ResHelper.GetString("dialogs.view.list.nodata");
            this.lblInfo.Visible = true;
        }

        // Reset pager control
        switch (this.ViewMode)
        {
            case DialogViewModeEnum.TilesView:
                pagerElemTiles.CurrentPage = 0;
                break;

            case DialogViewModeEnum.ThumbnailsView:
                pagerElemThumbnails.CurrentPage = 0;
                break;
        }

        base.OnPreRender(e);
    }


    protected void Page_Load(object sender, EventArgs e)
    {
        this.Visible = !this.StopProcessing;
        if (!this.StopProcessing)
        {
            if (UrlHelper.IsPostback())
            {
                Reload(true);
            }
        }
    }


    #region "Public methods"

    /// <summary>
    /// Loads view controls according currently set view mode.
    /// </summary>
    public void LoadViewControls()
    {
        this.InfoText = "";

        // Select mode according required
        switch (this.ViewMode)
        {
            case DialogViewModeEnum.ListView:
                this.plcListView.Visible = true;

                // Stop processing                
                this.plcTilesView.Visible = false;
                this.plcThumbnailsView.Visible = false;
                break;

            case DialogViewModeEnum.TilesView:
                this.plcTilesView.Visible = true;
                this.repTilesView.DataBindByDefault = false;
                pagerElemTiles.PageControl = repTilesView.ID;

                // Stop processing
                this.plcListView.Visible = false;
                this.plcThumbnailsView.Visible = false;
                break;

            case DialogViewModeEnum.ThumbnailsView:
                this.plcThumbnailsView.Visible = true;
                this.repThumbnailsView.DataBindByDefault = false;
                pagerElemThumbnails.PageControl = repThumbnailsView.ID;

                // Stop processing
                this.plcListView.Visible = false;
                this.plcTilesView.Visible = false;
                break;
        }
    }

    #endregion


    #region "Private methods"

    /// <summary>
    /// Initializes all nested controls.
    /// </summary>
    private void SetupControls()
    {
        // Initialize displaying controls according view mode
        LoadViewControls();

        InitializeControlScripts();

        // Select mode according required
        switch (this.ViewMode)
        {
            case DialogViewModeEnum.ListView:
                InitializeListView();
                this.ListViewControl.OnExternalDataBound += new OnExternalDataBoundEventHandler(ListViewControl_OnExternalDataBound);
                this.ListViewControl.GridView.RowDataBound += new GridViewRowEventHandler(GridView_RowDataBound);
                break;

            case DialogViewModeEnum.TilesView:
                InitializeTilesView();
                this.TilesViewControl.ItemDataBound += new RepeaterItemEventHandler(TilesViewControl_ItemDataBound);
                break;

            case DialogViewModeEnum.ThumbnailsView:
                InitializeThumbnailsView();
                this.ThumbnailsViewControl.ItemDataBound += new RepeaterItemEventHandler(ThumbnailsViewControl_ItemDataBound);
                break;
        }

        // Register delete confirmation script
        this.ltlScript.Text = ScriptHelper.GetScript("function DeleteConfirmation(){ return confirm('" + ResHelper.GetString("attach.deleteconfirmation") + "'); }");
    }


    /// <summary>
    /// Initializes all the necessary JavaScript blocks.
    /// </summary>
    private void InitializeControlScripts()
    {
        string script = @"  var activeColor = '#EDDEC7'; 
                            var deactiveColor = '#FFFFFF'; 
                            function ColorizeRow(itemId) {
                                if (itemId != null) 
                                {   
                                    var hdnField = document.getElementById('" + this.hdnItemToColorize.ClientID + "');" +
                         @"         if (hdnField != null) 
                                    {
                                        // If some item was previously selected
                                        if ((hdnField.value != null) && (hdnField.value != '')) 
                                        {   
                                            // Get selected item and reset its selection
                                            var lastColorizedElem = document.getElementById(hdnField.value);
                                            if (lastColorizedElem != null) 
                                            {   
                                                ColorizeElement(lastColorizedElem, deactiveColor, itemId);
                                            }
                                         }
                                        
                                         // Update field value
                                         hdnField.value = itemId;
                                     }                          

                                    // Colorize currently selected item
                                    var elem = document.getElementById(itemId);
                                    if (elem != null) 
                                    {
                                        ColorizeElement(elem, activeColor, itemId);
                                    }
                                 }
                             }

                             function ColorizeLastRow() {
                                var hdnField = document.getElementById('" + this.hdnItemToColorize.ClientID + "');" +
                        @"      if (hdnField != null) 
                                {
                                    // If some item was previously selected
                                    if ((hdnField.value != null) && (hdnField.value != '')) 
                                    {               
                                        // Get selected item and reset its selection
                                        var lastColorizedElem = document.getElementById(hdnField.value);
                                        if (lastColorizedElem != null) 
                                        {    
                                            ColorizeElement(lastColorizedElem, activeColor);
                                        }
                                    }
                                }
                            }

                            function ColorizeElement(elem, color, itemId) {
                                elem.style.backgroundColor = color;
                                var frames = elem.getElementsByTagName('iframe');
                                for (var i = 0; i <frames.length; i++) {
                                    var fBody = frames[i].contentWindow.document.body;
                                    if (fBody != null) {
                                        fBody.style.backgroundColor = color;
                                    }
                                    else if (itemId != null) {
                                        setTimeout(""ColorizeRow('""+itemId+""');"", 100);
                                    }
                                }
                            }
                          ";

        ScriptManager.RegisterStartupScript(this, this.GetType(), "DialogsColorize", script, true);

        // Dialog for editing image
        string url = "";
        if (this.SourceType == MediaSourceEnum.MediaLibraries)
        {
            if (this.IsLiveSite)
            {
                url = ResolveUrl("~/CMSModules/MediaLibrary/CMSPages/ImageEditor.aspx");
            }
            else
            {
                url = ResolveUrl("~/CMSModules/MediaLibrary/Controls/MediaLibrary/ImageEditor.aspx");
            }
        }
        else
        {
            if (this.IsLiveSite)
            {
                url = ResolveUrl("~/CMSFormControls/LiveSelectors/ImageEditor.aspx");
            }
            else
            {
                url = ResolveUrl("~/CMSDesk/Content/ImageEditor.aspx");
            }
        }

        string refreshType = CMSDialogHelper.GetMediaSource(this.SourceType);

        int parentId = QueryHelper.GetInteger("parentId", 0);

        string editImage = "function EditImage(param) {                                         " +
                "var form = ''; \n                                                              " +
                "modalDialog('" + url + "?clientid=" + HTMLHelper.HTMLEncode(refreshType) +
                ((parentId > 0) ? "&parentId=" + parentId : "") + "&refresh=1&refaction=0&' + param, 'imageEditorDialog', 900, 600); return false; }";

        ScriptManager.RegisterStartupScript(this, this.GetType(), "DialogsEditImage", editImage, true);
    }


    /// <summary>
    /// Loads data from the data source property.
    /// </summary>
    public void ReloadData()
    {
        // Select mode according required
        switch (this.ViewMode)
        {
            case DialogViewModeEnum.ListView:
                ReloadListView();
                break;

            case DialogViewModeEnum.TilesView:
                ReloadTilesView();
                break;

            case DialogViewModeEnum.ThumbnailsView:
                ReloadThumbnailsView();
                break;
        }
    }


    /// <summary>
    /// Reloads control with data.
    /// </summary>
    /// <param name="forceSetup">Indicates whether the inner controls should be re-setuped.</param>
    public void Reload(bool forceSetup)
    {
        this.Visible = !this.StopProcessing;
        if (this.Visible)
        {
            if (forceSetup)
            {
                // Initialize controls
                SetupControls();
            }

            // Load passed data
            ReloadData();
        }
    }


    /// <summary>
    /// Gets attachment ID based on the attachment type.
    /// </summary>
    /// <param name="drv">DataRowView object holding information on the attachment.</param>
    private int GetAttachmentID(DataRowView drv)
    {
        int result = 0;

        if (drv != null)
        {
            if (drv.DataView.Table.Columns.Contains("AttachmentID"))
            {
                result = ValidationHelper.GetInteger(drv["AttachmentID"], 0);
            }
            else
            {
                result = ValidationHelper.GetInteger(drv["AttachmentHistoryID"], 0);
            }
        }

        return result;
    }


    /// <summary>
    /// Returns the sitename according to item info.
    /// </summary>
    /// <param name="itemData">Row containing information on the current item.</param>
    /// <param name="isMediaFile">Indicates whether the file is media file.</param>
    private string GetSiteName(DataRow itemData, bool isMediaFile)
    {
        int siteId = 0;
        string result = "";

        if (isMediaFile)
        {
            siteId = ValidationHelper.GetInteger(itemData["FileSiteID"], 0);
        }
        else
        {
            if (itemData.Table.Columns.Contains("SiteName"))
            {
                result = ValidationHelper.GetString(itemData["SiteName"], "");
            }
            else
            {
                if (itemData.Table.Columns.Contains("AttachmentSiteID"))
                {
                    siteId = ValidationHelper.GetInteger(itemData["AttachmentSiteID"], 0);
                }
                else
                {
                    siteId = this.TreeNodeObj.NodeSiteID;
                }
            }
        }

        if (result == "")
        {
            if (!ContextStockHelper.Contains("DialogsSiteName") || (this.mLastSiteId != siteId))
            {
                SiteInfo si = SiteInfoProvider.GetSiteInfo(siteId);
                if (si != null)
                {
                    this.mLastSiteId = si.SiteID;
                    result = si.SiteName;
                    ContextStockHelper.Add("DialogsSiteName", result);
                }
            }
            else
            {
                result = ValidationHelper.GetString(ContextStockHelper.GetItem("DialogsSiteName"), "");
            }
        }

        return result;
    }


    /// <summary>
    /// Gets domain name of the site items are related to.
    /// </summary>
    /// <param name="siteId">ID of the site current item is related to.</param>
    private static object GetLastModified(int attachmentDocumentId)
    {
        object result = "";

        // Make sure information on site domain name is available in the context
        if (!ContextStockHelper.Contains("DialogsAttachmentLastModified"))
        {
            if (attachmentDocumentId > 0)
            {
                TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
                TreeNode node = DocumentHelper.GetDocument(attachmentDocumentId, tree);
                if (node != null)
                {
                    result = node.GetValue("DocumentModifiedWhen");
                }
            }

            // Save site name information for future use
            ContextStockHelper.Add("DialogsAttachmentLastModified", result);
        }
        else
        {
            // Get it from stock
            result = ContextStockHelper.GetItem("DialogsAttachmentLastModified");
        }

        return result;
    }


    /// <summary>
    /// Initializes attachment update control according current attachment data.
    /// </summary>
    /// <param name="drv">DataRowView holding attachment data.</param>
    private void GetAttachmentUpdateControl(ref CMSAdminControls_Attachments_DirectFileUploader_DirectFileUploader dfuElem, DataRowView drv)
    {
        if (dfuElem != null)
        {
            string refreshType = CMSDialogHelper.GetMediaSource(this.SourceType);
            //string siteName = CMSContext.CurrentSiteName;
            Guid formGuid = Guid.Empty;
            int documentId = ValidationHelper.GetInteger(drv["AttachmentDocumentID"], 0);

            // If attachment is related to the workflow 'AttachmentFormGUID' information isn't present
            if (drv.DataView.Table.Columns.Contains("AttachmentFormGUID"))
            {
                formGuid = ValidationHelper.GetGuid(drv["AttachmentFormGUID"], Guid.Empty);
            }

            dfuElem.ID = "dfuElem";
            dfuElem.ForceLoad = true;
            dfuElem.FormGUID = formGuid;
            dfuElem.DocumentID = documentId;

            if (this.TreeNodeObj != null)
            {
                // if attachment node exists
                dfuElem.NodeParentNodeID = this.TreeNodeObj.NodeParentID;
                dfuElem.NodeClassName = this.TreeNodeObj.NodeClassName;
            }
            else
            {
                // if attachment node doesn't exist
                dfuElem.NodeParentNodeID = this.NodeParentID;
                dfuElem.NodeClassName = "cms.file";
            }

            dfuElem.ControlGroup = "MediaView";
            dfuElem.AttachmentGUID = ValidationHelper.GetGuid(drv["AttachmentGUID"], Guid.Empty);
            dfuElem.ResizeToWidth = this.ResizeToWidth;
            dfuElem.ResizeToHeight = this.ResizeToHeight;
            dfuElem.ResizeToMaxSideSize = this.ResizeToMaxSideSize;
            dfuElem.AllowedExtensions = AllowedExtensions;
            dfuElem.ImageUrl = ResolveUrl(UrlHelper.ImagesDirectory + "/CMSAdminControls/UniGrid/upload.gif");
            dfuElem.ImageHeight = 16;
            dfuElem.ImageWidth = 16;
            dfuElem.InsertMode = false;
            dfuElem.ParentElemID = refreshType;
            dfuElem.IncludeNewItemInfo = true;
        }
    }


    /// <summary>
    /// Returns correct ID for the item (for colorizing the item when selected).
    /// </summary>
    /// <param name="dataItem">Container.DataItem</param>
    protected string GetID(object dataItem)
    {
        DataRowView dr = dataItem as DataRowView;
        if (dr != null)
        {
            return GetColorizeID(dr.Row);
        }
        return "";
    }


    /// <summary>
    /// Returns correct ID for the given item (for colorizing the item when selected).
    /// </summary>
    /// <param name="dr">Item to get the ID of</param>
    protected string GetColorizeID(DataRow dr)
    {
        string id = dr[this.FileIdColumn].ToString();
        if (String.IsNullOrEmpty(id))
        {
            id = dr["NodeGUID"].ToString();
        }
        return id;
    }

    #endregion


    #region "List view methods"

    /// <summary>
    /// Initializes list view controls.
    /// </summary>
    private void InitializeListView()
    {
        switch (this.SourceType)
        {
            case MediaSourceEnum.Content:
                this.gridList.OrderBy = "DocumentName";
                break;

            case MediaSourceEnum.MediaLibraries:
                this.gridList.OrderBy = "FileName";
                break;

            case MediaSourceEnum.DocumentAttachments:
                this.gridList.OrderBy = "AttachmentOrder, AttachmentName, AttachmentID";
                break;

            default:
                break;
        }
    }


    /// <summary>
    /// Reloads list view according source type.
    /// </summary>
    private void ReloadListView()
    {
        // Fill the grid data source
        if (!DataHelper.DataSourceIsEmpty(this.DataSource))
        {
            this.gridList.DataSource = this.DataSource;
            this.gridList.ReloadData();
        }
    }


    /// <summary>
    /// Returns panel with image according extension of the processed file. 
    /// </summary>
    /// <param name="ext">Extension of the file used to determine icon.</param>
    /// <param name="url">Url of the original image.</param>
    /// <param name="item">Control inserted as a file name.</param>
    /// <param name="width">Image width</param>
    /// <param name="height">Image height</param>
    private Panel GetListItem(string ext, string className, string url, string previewUrl, int width, int height, Control item, bool isSelectable)
    {
        Panel pnl = new Panel();
        pnl.CssClass = "DialogListItem" + (isSelectable ? "" : "Unselectable");
        pnl.Controls.Add(new LiteralControl("<div class=\"DialogListItemNameRow\">"));

        if ((className == "cms.file") || (className == ""))
        {
            // Generate image based of file extension
            InlineUserControl fileImage = this.Page.LoadControl("~/CMSInlineControls/ImageControl.ascx") as InlineUserControl;
            if (fileImage != null)
            {
                if (!isSelectable)
                {
                    fileImage.SetValue("Style", "cursor: default;");
                }
                fileImage.IsLiveSite = false;
                fileImage.SetValue("ShowFileIcons", true);
                fileImage.SetValue("Url", ResolveUrl(previewUrl));
                fileImage.SetValue("Extension", ext);
                if (ImageHelper.IsImage(ext))
                {
                    if ((width > 0) && (height > 0))
                    {
                        int[] dimensions = ImageHelper.EnsureImageDimensions(0, 0, 200, width, height);

                        fileImage.SetValue("Behavior", "hover");
                        fileImage.SetValue("MouseOverWidth", dimensions[0]);
                        fileImage.SetValue("MouseOverHeight", dimensions[1]);
                        fileImage.SetValue("ImageID", "Image" + idCount++);
                    }
                }
                if ((isSelectable) && (item is LinkButton))
                {
                    // Create clickabe compelte panel
                    pnl.Attributes["onclick"] = ((LinkButton)item).Attributes["onclick"];
                    ((LinkButton)item).Attributes["onclick"] = null;
                }
                pnl.Controls.Add(fileImage);
            }
        }
        else
        {
            Image docImg = new Image();

            docImg.ImageUrl = TreePathUtils.GetDocumentTypeIconUrl(className);
            docImg.Attributes["style"] = "width: 16px; height: 16px;";

            pnl.Controls.Add(docImg);

            if ((isSelectable) && (item is LinkButton))
            {
                // Create clickabe compelte panel
                pnl.Attributes["onclick"] = ((LinkButton)item).Attributes["onclick"];
                ((LinkButton)item).Attributes["onclick"] = null;
            }
        }

        // Add file name                  
        pnl.Controls.Add(new LiteralControl("&nbsp;<span class=\"DialogListItemName\" " + (!isSelectable ? "style=\"cursor:default;\"" : "") + ">"));
        pnl.Controls.Add(item);
        pnl.Controls.Add(new LiteralControl("</span></div>"));

        return pnl;
    }


    protected void GridView_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        if (e.Row.RowType == DataControlRowType.DataRow)
        {
            DataRowView drv = (e.Row.DataItem as DataRowView);
            if (drv != null)
            {
                e.Row.Attributes["id"] = GetColorizeID(drv.Row);
            }
        }
    }


    protected object ListViewControl_OnExternalDataBound(object sender, string sourceName, object parameter)
    {
        object result = null;

        string argument = "";
        string url = "";

        bool isContent = (this.SourceType == MediaSourceEnum.Content);
        bool notAttachment = false;
        bool isContentFile = false;
        bool isSelectable = false;

        switch (sourceName.ToLower())
        {
            case "select":
                GridViewRow gvr = (parameter as GridViewRow);
                DataRowView drv = (DataRowView)gvr.DataItem;
                ImageButton btn = ((ImageButton)sender);

                string className = "";

                isContentFile = isContent ? (drv["ClassName"].ToString().ToLower() == "cms.file") : false;

                // Is current item CMS.File or attachment ?
                notAttachment = isContent && !(isContentFile && (ValidationHelper.GetInteger(drv["AttachmentID"], 0) != 0));
                if (notAttachment)
                {
                    className = DataClassInfoProvider.GetDataClass((int)drv["NodeClassID"]).ClassDisplayName;
                }

                string ext = HTMLHelper.HTMLEncode(notAttachment ? className : drv[this.FileExtensionColumn].ToString().TrimStart('.'));

                isSelectable = CMSDialogHelper.IsItemSelectable(this.SelectableContent, ext, isContentFile);

                // Check if item is selectable, if not remove select action button
                if (!isSelectable)
                {
                    btn.ImageUrl = ResolveUrl(UrlHelper.ImagesDirectory + "/CMSAdminControls/UniGrid/transparent.gif");
                    btn.ToolTip = "";
                    btn.Attributes["style"] = "margin:0px 3px;cursor:default;";
                    btn.Enabled = false;
                }
                else
                {
                    argument = RaiseOnGetArgumentSet(drv.Row);
                    url = RaiseOnGetListItemUrl(drv.Row, false, notAttachment);

                    // Initialize command
                    btn.Attributes["onclick"] = "ColorizeRow('" + GetColorizeID(drv.Row) + "'); SetSelectAction(\"" + argument + ";" + url + "\"); return false;";

                    result = btn;
                }
                break;

            case "selectsubdocs":
                drv = (DataRowView)(parameter as GridViewRow).DataItem;
                btn = ((ImageButton)sender);

                int nodeId = ValidationHelper.GetInteger(drv["NodeID"], 0);
                int nodeChildCount = ValidationHelper.GetInteger(drv["NodeChildNodesCount"], 0);

                if (this.IsDisplayMore)
                {
                    // Check if item is selectable, if not remove select action button
                    if (nodeChildCount > 0)
                    {
                        // Initialize command
                        btn.Attributes["onclick"] = "SetParentAction('" + nodeId + "'); return false;";
                    }
                    else
                    {
                        btn.ImageUrl = ResolveUrl(UrlHelper.ImagesDirectory + "/CMSAdminControls/UniGrid/subdocumentdisabled.gif");
                        btn.ToolTip = ResHelper.GetString("dialogs.list.actions.showsubdocuments");
                        btn.Attributes["style"] = "margin:0px 3px;cursor:default;";
                        btn.Enabled = false;
                    }
                }
                else
                {
                    btn.Attributes["style"] = "display:none;";
                }
                break;

            case "view":
                gvr = (parameter as GridViewRow);
                drv = (DataRowView)gvr.DataItem;

                // Initialize VIEW button
                btn = ((ImageButton)sender);

                // Is current item CMS.File or attachment ?
                isContentFile = isContent ? (drv["ClassName"].ToString().ToLower() == "cms.file") : false;
                notAttachment = isContent && !(isContentFile && (ValidationHelper.GetInteger(drv["AttachmentID"], 0) != 0));
                if (!notAttachment)
                {
                    // Get media file URL according system settings
                    argument = RaiseOnGetArgumentSet(drv.Row);
                    url = RaiseOnGetListItemUrl(drv.Row, false, notAttachment);

                    btn.OnClientClick = "javascript: window.open('" + UrlHelper.ResolveUrl(url) + "'); return false;";
                }
                else
                {
                    btn.Visible = false;
                }
                break;

            case "edit":
                gvr = (parameter as GridViewRow);
                drv = (DataRowView)gvr.DataItem;

                isContentFile = isContent ? (drv["ClassName"].ToString().ToLower() == "cms.file") : false;
                notAttachment = isContent && !(isContentFile && (ValidationHelper.GetInteger(drv["AttachmentID"], 0) != 0));

                // Get file extension
                ext = (notAttachment ? "" : drv[this.FileExtensionColumn].ToString().TrimStart('.'));
                Guid guid = Guid.Empty;
                ImageButton imgEdit = (ImageButton)sender;
                if (this.SourceType == MediaSourceEnum.MediaLibraries)
                {
                    int siteId = ValidationHelper.GetInteger(drv["FileSiteID"], 0);
                    guid = ValidationHelper.GetGuid(drv["FileGUID"], Guid.Empty);
                    imgEdit.AlternateText = ext + ";MediaFileGUID=" + guid + "&sitename=" + GetSiteName(drv.Row, true);
                    imgEdit.PreRender += new EventHandler(img_PreRender);
                    imgEdit.ImageUrl = ResolveUrl(UrlHelper.ImagesDirectory + "/CMSAdminControls/Dialogs/edit.gif");
                }
                else if (!notAttachment)
                {
                    nodeId = ValidationHelper.GetInteger(drv["NodeID"], 0);

                    string nodeIdQuery = "";
                    if (this.SourceType == MediaSourceEnum.Content)
                    {
                        nodeIdQuery = "&nodeId=" + drv["NodeID"].ToString();
                    }

                    // Get the node workflow
                    this.VersionHistoryID = ValidationHelper.GetInteger(drv["DocumentCheckedOutVersionHistoryID"], 0);

                    guid = ValidationHelper.GetGuid(drv["AttachmentGUID"], Guid.Empty);
                    imgEdit.AlternateText = ext + ";AttachmentGUID=" + guid + "&sitename=" + GetSiteName(drv.Row, false) + nodeIdQuery + "&versionHistoryId=" + this.VersionHistoryID;
                    imgEdit.PreRender += new EventHandler(img_PreRender);
                    imgEdit.ImageUrl = ResolveUrl(UrlHelper.ImagesDirectory + "/CMSAdminControls/Dialogs/edit.gif");
                }
                else
                {
                    imgEdit.Visible = false;
                }
                break;

            case "name":
                drv = (DataRowView)parameter;

                string fileNameColumn = this.FileNameColumn;

                // Is current item CMS.File or attachment ?
                isContentFile = isContent ? (drv["ClassName"].ToString().ToLower() == "cms.file") : false;
                notAttachment = isContent && !(isContentFile && (ValidationHelper.GetInteger(drv["AttachmentID"], 0) != 0));
                if (notAttachment)
                {
                    fileNameColumn = "DocumentName";
                }

                // Get name and extension
                ext = (notAttachment ? "" : drv[this.FileExtensionColumn].ToString().TrimStart('.'));
                string name = HTMLHelper.HTMLEncode(drv[fileNameColumn].ToString());
                int width = ValidationHelper.GetInteger(drv[this.FileWidthColumn], 0);
                int height = ValidationHelper.GetInteger(drv[this.FileHeightColumn], 0);

                string cmpltExt = (notAttachment ? "" : "." + ext);
                string fileName = (cmpltExt != "") ? name.Replace(cmpltExt, "") : name;
                if (isContent && !notAttachment)
                {
                    string attachmentName = Path.GetFileNameWithoutExtension(drv["AttachmentName"].ToString());
                    if (fileName.ToLower() != attachmentName.ToLower())
                    {
                        fileName += " (" + HTMLHelper.HTMLEncode(drv["AttachmentName"].ToString()) + ")";
                    }
                }

                className = isContent ? HTMLHelper.HTMLEncode(drv["ClassName"].ToString().ToLower()) : "";

                isContentFile = (className == "cms.file");

                // Check if item is selectable
                if (!CMSDialogHelper.IsItemSelectable(this.SelectableContent, ext, isContentFile))
                {
                    LiteralControl ltlName = new LiteralControl(fileName);

                    // Get final panel
                    result = GetListItem(ext, className, "", "", width, height, ltlName, false);
                }
                else
                {
                    // Make a file name link
                    LinkButton lnkBtn = new LinkButton();

                    // Escape chars for postback javascript event
                    lnkBtn.ID = fileName.Replace("'", "").Replace("$", "");
                    lnkBtn.Text = HTMLHelper.HTMLEncode(fileName);

                    // Is current item CMS.File or attachment ?
                    isContentFile = isContent ? (drv["ClassName"].ToString().ToLower() == "cms.file") : false;
                    notAttachment = isContent && !(isContentFile && (ValidationHelper.GetInteger(drv["AttachmentID"], 0) != 0));

                    argument = RaiseOnGetArgumentSet(drv.Row);
                    url = RaiseOnGetListItemUrl(drv.Row, false, notAttachment);
                    string previewUrl = RaiseOnGetListItemUrl(drv.Row, true, notAttachment);

                    // Initialize command
                    lnkBtn.Attributes["onclick"] = "ColorizeRow('" + GetColorizeID(drv.Row) + "'); SetSelectAction(\"" + argument + ";" + url + "\"); return false;";

                    // Get final panel
                    result = GetListItem(ext, className, url, previewUrl, width, height, lnkBtn, true);
                }
                break;

            case "type":
                drv = (DataRowView)parameter;

                if (isContent)
                {
                    // Is current item CMS.File or attachment ?
                    isContentFile = isContent ? (drv["ClassName"].ToString().ToLower() == "cms.file") : false;
                    notAttachment = isContent && !(isContentFile && (ValidationHelper.GetInteger(drv["AttachmentID"], 0) != 0));
                    if (notAttachment || (this.OutputFormat == OutputFormatEnum.HTMLLink) || (this.OutputFormat == OutputFormatEnum.BBLink))
                    {
                        return HTMLHelper.HTMLEncode(drv["ClassDisplayName"].ToString());
                    }
                }
                result = drv[this.FileExtensionColumn].ToString().ToLower().Trim('.');
                break;

            case "size":
                drv = (DataRowView)parameter;

                // Is current item CMS.File or attachment ?
                isContentFile = isContent ? (drv["ClassName"].ToString().ToLower() == "cms.file") : false;
                notAttachment = isContent && !(isContentFile && (ValidationHelper.GetInteger(drv["AttachmentID"], 0) != 0));
                if (!notAttachment)
                {
                    long size = ValidationHelper.GetLong(drv[this.FileSizeColumn], 0);
                    result = DataHelper.GetSizeString(size);
                }
                break;

            case "attachmentmodified":
                drv = (DataRowView)parameter;

                // If attachment is related to the workflow 'AttachmentLastModified' information isn't present
                if (!drv.DataView.Table.Columns.Contains("AttachmentLastModified"))
                {
                    int attachmentDocumentId = ValidationHelper.GetInteger(drv["AttachmentDocumentID"], 0);

                    result = GetLastModified(attachmentDocumentId);
                }
                else
                {
                    result = drv["AttachmentLastModified"];
                }
                break;

            case "attachmentupdate":
                drv = (DataRowView)parameter;

                // Dynamically load uploader control
                CMSAdminControls_Attachments_DirectFileUploader_DirectFileUploader dfuElem = Page.LoadControl("~/CMSAdminControls/Attachments/DirectFileUploader/DirectFileUploader.ascx") as CMSAdminControls_Attachments_DirectFileUploader_DirectFileUploader;

                Panel updatePanel = new Panel();
                updatePanel.Attributes["style"] = "text-align:center;";

                // Initialize update control
                GetAttachmentUpdateControl(ref dfuElem, drv);

                updatePanel.Controls.Add(dfuElem);

                result = updatePanel;
                break;

            case "attachmentdelete":
                gvr = (parameter as GridViewRow);
                drv = (DataRowView)gvr.DataItem;

                // Initialize DELETE button
                btn = ((ImageButton)sender);
                btn.OnClientClick = "if(DeleteConfirmation() == false){return false;} SetAction('attachmentdelete', '" + drv[this.FileIdColumn].ToString() + "'); RaiseHiddenPostBack(); return false;";
                break;

            case "attachmentmoveup":
                gvr = (parameter as GridViewRow);
                drv = (DataRowView)gvr.DataItem;

                // Initialize MOVE UP button
                btn = ((ImageButton)sender);

                // Get attachment ID
                int attachmentId = GetAttachmentID(drv);

                btn.OnClientClick = "SetAction('attachmentmoveup', '" + attachmentId.ToString() + "'); RaiseHiddenPostBack(); return false;";
                break;

            case "attachmentmovedown":
                gvr = (parameter as GridViewRow);
                drv = (DataRowView)gvr.DataItem;

                // Initialize MOVE DOWN button
                btn = ((ImageButton)sender);

                // Get attachment ID
                attachmentId = GetAttachmentID(drv);

                btn.OnClientClick = "SetAction('attachmentmovedown', '" + attachmentId.ToString() + "'); RaiseHiddenPostBack(); return false;";
                break;

            case "attachmentedit":
                gvr = (parameter as GridViewRow);
                drv = (DataRowView)gvr.DataItem;

                // Get file extension
                string attExtension = ValidationHelper.GetString(drv["AttachmentExtension"], "").ToLower();
                Guid attGuid = ValidationHelper.GetGuid(drv["AttachmentGUID"], Guid.Empty);

                if ((this.TreeNodeObj != null) && (this.VersionHistoryID == 0))
                {
                    // Get the node workflow
                    WorkflowManager wm = new WorkflowManager(this.TreeNodeObj.TreeProvider);
                    WorkflowInfo wi = wm.GetNodeWorkflow(this.TreeNodeObj);
                    if (wi != null)
                    {
                        // Ensure the document version
                        VersionManager vm = new VersionManager(this.TreeNodeObj.TreeProvider);
                        this.VersionHistoryID = vm.EnsureVersion(this.TreeNodeObj, this.TreeNodeObj.IsPublished);
                    }
                }

                ImageButton attImg = (ImageButton)sender;
                attImg.AlternateText = attExtension + ";AttachmentGUID=" + attGuid + "&sitename=" + GetSiteName(drv.Row, false) + "&versionHistoryId=" + this.VersionHistoryID;
                attImg.PreRender += new EventHandler(img_PreRender);
                break;
        }

        return result;
    }


    protected void img_PreRender(object sender, EventArgs e)
    {
        ImageButton img = (ImageButton)sender;

        string[] args = img.AlternateText.Split(';');
        if (args.Length == 2)
        {
            // If the file is not an image don't allow image editing
            if (!ImageHelper.IsSupportedByImageEditor(args[0]))
            {
                // Disable edit icon in case that attachment is not an image
                img.ImageUrl = ResolveUrl(UrlHelper.ImagesDirectory + "/CMSAdminControls/UniGrid/editdisabled.gif");
                img.Enabled = false;
                img.Style.Add("cursor", "default");
            }
            else
            {
                img.Attributes["onclick"] = "if(!($(this).hasClass('Edited'))){ EditImage(\"" + args[1] + "\"); } return false;";
            }

            img.ToolTip = ResHelper.GetString("general.edit");
            img.AlternateText = ResHelper.GetString("general.edit");
        }
    }

    #endregion


    #region "Tiles view methods"

    /// <summary>
    /// Initializes controls for the tiles view mode.
    /// </summary>
    private void InitializeTilesView()
    {
        // Initialize page size
        bool isAttachmentTab = this.SourceType == MediaSourceEnum.DocumentAttachments;
        if (isAttachmentTab)
        {
            this.pageSizeTiles.Items = new string[] { "20", "40", "100" };
        }
        else
        {
            this.pageSizeTiles.Items = new string[] { "16", "32", "100" };
        }
        pagerElemTiles.PageSize = (this.pageSizeTiles.SelectedValue == "-1") ? 0 : ValidationHelper.GetInteger(this.pageSizeTiles.SelectedValue, (isAttachmentTab ? 20 : 16));

        // Basic control properties
        repTilesView.HideControlForZeroRows = true;

        // UniPager properties        
        pagerElemTiles.GroupSize = PAGER_GROUP_SIZE;
        pagerElemTiles.DisplayFirstLastAutomatically = false;
        pagerElemTiles.DisplayPreviousNextAutomatically = false;
        pagerElemTiles.HidePagerForSinglePage = true;
        pagerElemTiles.PagerMode = UniPagerMode.PostBack;
    }


    /// <summary>
    /// Loads content for media libraries tile view element.
    /// </summary>
    private void ReloadTilesView()
    {
        // Connects repeater with data source
        if (!DataHelper.DataSourceIsEmpty(this.DataSource))
        {
            repTilesView.DataSource = this.DataSource;
            repTilesView.DataBind();
        }
    }


    protected void TilesViewControl_ItemDataBound(object sender, RepeaterItemEventArgs e)
    {
        string imageBaseUrl = UrlHelper.ImagesDirectory + "/CMSAdminControls/UniGrid/";

        string className = "";
        string fileNameColumn = this.FileNameColumn;

        DataRow dr = ((DataRowView)e.Item.DataItem).Row;

        bool isContent = (this.SourceType == MediaSourceEnum.Content);

        bool isContentFile = isContent ? (dr["ClassName"].ToString().ToLower() == "cms.file") : false;
        bool notAttachment = isContent && !(isContentFile && (dr["AttachmentGUID"] != DBNull.Value));
        if (notAttachment)
        {
            className = DataClassInfoProvider.GetDataClass((int)dr["NodeClassID"]).ClassDisplayName;

            fileNameColumn = "DocumentName";
        }
        else
        {
            fileNameColumn = "AttachmentName";
        }

        // Get basic information on file (use field properties for CMS.File | attachment, document columns otherwise)
        string fileName = HTMLHelper.HTMLEncode((dr.Table.Columns.Contains(fileNameColumn) ? dr[fileNameColumn].ToString() : dr[this.FileNameColumn].ToString()));
        string ext = HTMLHelper.HTMLEncode(notAttachment ? className : dr[this.FileExtensionColumn].ToString().TrimStart('.'));
        string argument = RaiseOnGetArgumentSet(dr);
        int width = ValidationHelper.GetInteger(dr[this.FileWidthColumn], 0);
        int height = ValidationHelper.GetInteger(dr[this.FileHeightColumn], 0);

        // Get item image URL from the parent set
        string url = RaiseOnGetTilesThumbsItemUrl(dr, true, 0, 0, 48, notAttachment);
        string selectUrl = RaiseOnGetTilesThumbsItemUrl(dr, false, 0, 0, 0, notAttachment);
        bool isSelectable = CMSDialogHelper.IsItemSelectable(this.SelectableContent, ext, isContentFile);

        #region "Load file image"

        // Selectable area
        Panel pnlItemInageContainer = e.Item.FindControl("pnlTiles") as Panel;
        if (pnlItemInageContainer != null)
        {
            if (isSelectable)
            {
                pnlItemInageContainer.Attributes["onclick"] = "ColorizeRow('" + GetColorizeID(dr) + "'); SetSelectAction(\"" + argument + ";" + selectUrl + "\"); return false;";
                pnlItemInageContainer.Attributes["style"] = "cursor:pointer;";
            }
            else
            {
                pnlItemInageContainer.Attributes["style"] = "cursor:default;";
            }
        }

        // Image area
        InlineUserControl fileImage = e.Item.FindControl("imgElem") as InlineUserControl;
        if (fileImage != null)
        {
            fileImage.SetValue("Url", ResolveUrl(url));
            fileImage.SetValue("Tooltip", fileName);
            fileImage.SetValue("Alt", TextHelper.LimitLength(fileName, 8));
            fileImage.SetValue("Extension", ext);
            fileImage.SetValue("Style", "border: none");

            if (ImageHelper.IsImage(ext))
            {
                if ((width > 0) && (height > 0))
                {
                    int[] dimensions = ImageHelper.EnsureImageDimensions(0, 0, 200, width, height);

                    fileImage.SetValue("Behavior", "hover");
                    fileImage.SetValue("MouseOverWidth", dimensions[0]);
                    fileImage.SetValue("MouseOverHeight", dimensions[1]);
                    fileImage.SetValue("ImageID", "Image" + idCount++);
                }
            }
        }

        #endregion

        bool hideDocName = false;

        Label lblDocumentName = null;
        if (isContent && !notAttachment)
        {
            string docName = dr[this.FileNameColumn].ToString();
            fileName = Path.GetFileNameWithoutExtension(fileName);

            hideDocName = (docName.ToLower() == fileName.ToLower());
            if (!hideDocName)
            {
                fileName += "." + ext;

                lblDocumentName = e.Item.FindControl("lblDocumentName") as Label;
                if (lblDocumentName != null)
                {
                    lblDocumentName.Attributes["class"] = "DialogTileTitleBold";
                    lblDocumentName.Text = HTMLHelper.HTMLEncode(docName);
                }
            }
            else
            {
                fileName = docName;
            }
        }

        // Do not display document name if the same as the file name
        if (hideDocName || (lblDocumentName == null) || string.IsNullOrEmpty(lblDocumentName.Text))
        {
            PlaceHolder plcDocumentName = e.Item.FindControl("plcDocumentName") as PlaceHolder;
            if (plcDocumentName != null)
            {
                plcDocumentName.Visible = false;
            }
        }

        // Load file name
        Label lblFileName = e.Item.FindControl("lblFileName") as Label;
        if (lblFileName != null)
        {
            lblFileName.Text = fileName;
        }

        // Load file type
        Label lblType = e.Item.FindControl("lblTypeValue") as Label;
        if (lblType != null)
        {
            lblType.Text = ext.ToLower().Trim('.');
        }

        if (!notAttachment)
        {
            // Load file size
            Label lblSize = e.Item.FindControl("lblSizeValue") as Label;
            if (lblSize != null)
            {
                long size = ValidationHelper.GetLong(dr[this.FileSizeColumn], 0);
                lblSize.Text = DataHelper.GetSizeString(size);
            }
        }

        // Initialize SELECT button
        ImageButton btnSelect = e.Item.FindControl("btnSelect") as ImageButton;
        if (btnSelect != null)
        {
            // Check if item is selectable, if not remove select action button
            if (!isSelectable)
            {
                btnSelect.ImageUrl = ResolveUrl(imageBaseUrl + "transparent.gif");
                btnSelect.ToolTip = "";
                btnSelect.Attributes.Remove("onclick");
                btnSelect.Attributes["style"] = "cursor:default;";
                btnSelect.Enabled = false;
            }
            else
            {
                btnSelect.ImageUrl = ResolveUrl(imageBaseUrl + "next.gif");
                btnSelect.ToolTip = ResHelper.GetString("dialogs.list.actions.select");

                // Initialize command
                btnSelect.Attributes["onclick"] = "ColorizeRow('" + GetColorizeID(dr) + "'); SetSelectAction(\"" + argument + ";" + selectUrl + "\"); return false;";
            }
        }

        // Initialize SELECTSUBDOCS button
        ImageButton btnSelectSubDocs = e.Item.FindControl("btnSelectSubDocs") as ImageButton;
        if (btnSelectSubDocs != null)
        {
            btnSelectSubDocs.ToolTip = ResHelper.GetString("dialogs.list.actions.showsubdocuments");

            if (this.IsDisplayMore && (this.SourceType == MediaSourceEnum.Content))
            {
                int nodeId = ValidationHelper.GetInteger(dr["NodeID"], 0);
                int nodeChildCount = ValidationHelper.GetInteger(dr["NodeChildNodesCount"], 0);

                // Check if item is selectable, if not remove select action button
                if (nodeChildCount > 0)
                {
                    // Initialize command
                    btnSelectSubDocs.Attributes["onclick"] = "SetParentAction('" + nodeId + "'); return false;";
                    btnSelectSubDocs.ImageUrl = ResolveUrl(UrlHelper.ImagesDirectory + "/CMSAdminControls/UniGrid/subdocument.gif");

                }
                else
                {
                    btnSelectSubDocs.ImageUrl = ResolveUrl(UrlHelper.ImagesDirectory + "/CMSAdminControls/UniGrid/subdocumentdisabled.gif");
                    btnSelectSubDocs.Attributes["style"] = "cursor:default;";
                    btnSelectSubDocs.Enabled = false;
                }
            }
            else
            {
                btnSelectSubDocs.Visible = false;
            }
        }

        // Initialize VIEW button
        // Get media file URL according system settings
        argument = RaiseOnGetArgumentSet(dr);

        ImageButton btnView = e.Item.FindControl("btnView") as ImageButton;
        if (btnView != null)
        {
            if (!notAttachment)
            {
                btnView.ImageUrl = ResolveUrl(imageBaseUrl + "view.gif");
                btnView.ToolTip = ResHelper.GetString("dialogs.list.actions.view");
                btnView.OnClientClick = "javascript: window.open('" + UrlHelper.ResolveUrl(selectUrl) + "'); return false;";
            }
            else
            {
                btnView.Visible = false;
            }
        }

        // Initialize EDIT button
        ImageButton btnContentEdit = e.Item.FindControl("btnContentEdit") as ImageButton;
        if (btnContentEdit != null)
        {
            btnContentEdit.ToolTip = ResHelper.GetString("general.edit");

            Guid guid = Guid.Empty;
            if (this.SourceType == MediaSourceEnum.MediaLibraries)
            {
                guid = ValidationHelper.GetGuid(dr["FileGUID"], Guid.Empty);
                btnContentEdit.ImageUrl = ResolveUrl(UrlHelper.ImagesDirectory + "/CMSAdminControls/Dialogs/edit.gif");
                btnContentEdit.AlternateText = ext + ";MediaFileGUID=" + guid + "&sitename=" + GetSiteName(dr, true);
                btnContentEdit.PreRender += new EventHandler(img_PreRender);
            }
            else if (!notAttachment)
            {
                string nodeid = "";
                if (this.SourceType == MediaSourceEnum.Content)
                {
                    nodeid = "&nodeId=" + dr["NodeID"].ToString();

                    // Get the node workflow
                    this.VersionHistoryID = ValidationHelper.GetInteger(dr["DocumentCheckedOutVersionHistoryID"], 0);
                }

                guid = ValidationHelper.GetGuid(dr["AttachmentGUID"], Guid.Empty);
                btnContentEdit.ImageUrl = ResolveUrl(UrlHelper.ImagesDirectory + "/CMSAdminControls/Dialogs/edit.gif");
                btnContentEdit.AlternateText = ext + ";AttachmentGUID=" + guid + "&sitename=" + GetSiteName(dr, false) + nodeid +
                    ((this.VersionHistoryID > 0) ? "&versionHistoryId=" + this.VersionHistoryID : "");
                btnContentEdit.PreRender += new EventHandler(img_PreRender);
            }
            else
            {
                btnContentEdit.Visible = false;
            }
        }

        // If attachments being displayed show additional actions
        if (this.SourceType == MediaSourceEnum.DocumentAttachments)
        {
            // Initialize UPDATE button
            CMSAdminControls_Attachments_DirectFileUploader_DirectFileUploader dfuElem = e.Item.FindControl("dfuElem") as CMSAdminControls_Attachments_DirectFileUploader_DirectFileUploader;
            if (dfuElem != null)
            {
                DataRowView drv = (DataRowView)e.Item.DataItem;

                GetAttachmentUpdateControl(ref dfuElem, drv);
            }

            // Initialize EDIT button
            ImageButton btnEdit = e.Item.FindControl("btnEdit") as ImageButton;
            if (btnEdit != null)
            {
                if (!notAttachment)
                {
                    btnEdit.ToolTip = ResHelper.GetString("general.edit");

                    // Get file extension
                    string extension = ValidationHelper.GetString(dr["AttachmentExtension"], "").ToLower();
                    Guid guid = ValidationHelper.GetGuid(dr["AttachmentGUID"], Guid.Empty);

                    // Get the node workflow
                    if ((this.TreeNodeObj != null) && (this.VersionHistoryID == 0))
                    {
                        // Get the node workflow
                        WorkflowManager wm = new WorkflowManager(this.TreeNodeObj.TreeProvider);
                        WorkflowInfo wi = wm.GetNodeWorkflow(this.TreeNodeObj);
                        if (wi != null)
                        {
                            // Ensure the document version
                            VersionManager vm = new VersionManager(this.TreeNodeObj.TreeProvider);
                            this.VersionHistoryID = vm.EnsureVersion(this.TreeNodeObj, this.TreeNodeObj.IsPublished);
                        }
                    }

                    btnEdit.ImageUrl = ResolveUrl(imageBaseUrl + "edit.gif");
                    btnEdit.AlternateText = extension + ";AttachmentGUID=" + guid + "&sitename=" + GetSiteName(dr, false) + "&versionHistoryId=" + this.VersionHistoryID;
                    btnEdit.PreRender += new EventHandler(img_PreRender);
                }
            }

            // Initialize DELETE button
            ImageButton btnDelete = e.Item.FindControl("btnDelete") as ImageButton;
            if (btnDelete != null)
            {
                btnDelete.ImageUrl = ResolveUrl(imageBaseUrl + "delete.gif");
                btnDelete.ToolTip = ResHelper.GetString("general.delete");

                // Initialize command
                btnDelete.Attributes["onclick"] = "if(DeleteConfirmation() == false){return false;} SetAction('attachmentdelete','" + dr["AttachmentGUID"] + "'); RaiseHiddenPostBack(); return false;";
            }

            PlaceHolder plcContentEdit = e.Item.FindControl("plcContentEdit") as PlaceHolder;
            plcContentEdit.Visible = false;
        }
        else
        {
            PlaceHolder plcAttachmentActions = e.Item.FindControl("plcAttachmentActions") as PlaceHolder;
            plcAttachmentActions.Visible = false;
        }
    }

    #endregion


    #region "Thumbnails view methods"

    /// <summary>
    /// Initializes controls for the thumbnails view mode.
    /// </summary>
    private void InitializeThumbnailsView()
    {
        bool isAttachmentTab = this.SourceType == MediaSourceEnum.DocumentAttachments;

        // Initialize page size
        if (isAttachmentTab)
        {
            this.pageSizeThumbs.Items = new string[] { "12", "24", "48", "96" };
        }
        else
        {
            this.pageSizeThumbs.Items = new string[] { "10", "20", "50", "100" };
        }
        pagerElemThumbnails.PageSize = (this.pageSizeThumbs.SelectedValue == "-1") ? 0 : ValidationHelper.GetInteger(this.pageSizeThumbs.SelectedValue, (isAttachmentTab ? 12 : 10));

        // Basic control properties
        repThumbnailsView.HideControlForZeroRows = true;

        // UniPager properties        
        pagerElemThumbnails.GroupSize = PAGER_GROUP_SIZE;
        pagerElemThumbnails.DisplayFirstLastAutomatically = false;
        pagerElemThumbnails.DisplayPreviousNextAutomatically = false;
        pagerElemThumbnails.HidePagerForSinglePage = true;
        pagerElemThumbnails.PagerMode = UniPagerMode.PostBack;
    }


    /// <summary>
    /// Loads content for media libraries thumbnails view element.
    /// </summary>
    private void ReloadThumbnailsView()
    {
        // Connects repeater with data source
        if (!DataHelper.DataSourceIsEmpty(this.DataSource))
        {
            repThumbnailsView.DataSource = this.DataSource;
            repThumbnailsView.DataBind();
        }
    }


    protected void ThumbnailsViewControl_ItemDataBound(object sender, RepeaterItemEventArgs e)
    {
        string imageBaseUrl = UrlHelper.ImagesDirectory + "/CMSAdminControls/UniGrid/";

        DataRow dr = ((DataRowView)e.Item.DataItem).Row;

        string fileNameColumn = this.FileNameColumn;
        string className = "";

        bool isContent = (this.SourceType == MediaSourceEnum.Content);

        bool isContentFile = isContent ? (dr["ClassName"].ToString().ToLower() == "cms.file") : false;
        bool notAttachment = isContent && !(isContentFile && (dr["AttachmentGUID"] != DBNull.Value));
        if (notAttachment)
        {
            className = DataClassInfoProvider.GetDataClass((int)dr["NodeClassID"]).ClassDisplayName;

            fileNameColumn = "DocumentName";
        }

        // Get information on file
        string fileName = HTMLHelper.HTMLEncode(dr[fileNameColumn].ToString());
        string ext = HTMLHelper.HTMLEncode(notAttachment ? className : dr[this.FileExtensionColumn].ToString().TrimStart('.'));
        string argument = RaiseOnGetArgumentSet(dr);

        // Get thumb preview image dimensions
        int[] thumbImgDimension = new int[] { 0, 0 };
        if (ImageHelper.IsImage(ext))
        {
            int origWidth = ValidationHelper.GetInteger(dr[this.FileWidthColumn], 0);
            int origHeight = ValidationHelper.GetInteger(dr[this.FileHeightColumn], 0);

            thumbImgDimension = CMSDialogHelper.GetThumbImageDimensions(origHeight, origWidth, MaxThumbImgHeight, MaxThumbImgWidth);
        }

        string url = RaiseOnGetTilesThumbsItemUrl(dr, true, thumbImgDimension[0], thumbImgDimension[1], 0, notAttachment);
        string selectUrl = RaiseOnGetTilesThumbsItemUrl(dr, false, 0, 0, 0, notAttachment);
        bool isSelectable = CMSDialogHelper.IsItemSelectable(this.SelectableContent, ext, isContentFile);

        #region "Load file image"

        // Selectable area
        Panel pnlItemInageContainer = e.Item.FindControl("pnlThumbnails") as Panel;
        if (pnlItemInageContainer != null)
        {
            if (isSelectable)
            {
                pnlItemInageContainer.Attributes["onclick"] = "ColorizeRow('" + GetColorizeID(dr) + "'); SetSelectAction(\"" + argument + ";" + selectUrl + "\"); return false;";
                pnlItemInageContainer.Attributes["style"] = "cursor:pointer;";
            }
            else
            {
                pnlItemInageContainer.Attributes["style"] = "cursor:default;";
            }
        }

        // Image area
        Image imgFile = e.Item.FindControl("imgFile") as Image;
        if (imgFile != null)
        {
            imgFile.ImageUrl = url;
            imgFile.AlternateText = TextHelper.LimitLength(fileName, 10);
            imgFile.Attributes["title"] = fileName.Replace("\"", "\\\"");
        }

        #endregion

        // Load file name
        Label lblName = e.Item.FindControl("lblFileName") as Label;
        if (lblName != null)
        {
            lblName.Text = fileName;
        }

        // Initialize SELECT button
        ImageButton btnSelect = e.Item.FindControl("btnSelect") as ImageButton;
        if (btnSelect != null)
        {
            // Check if item is selectable, if not remove select action button
            if (!isSelectable)
            {
                btnSelect.ImageUrl = ResolveUrl(imageBaseUrl + "transparent.gif");
                btnSelect.ToolTip = "";
                btnSelect.Attributes.Remove("onclick");
                btnSelect.Attributes["style"] = "cursor: default;";
                btnSelect.Enabled = false;
            }
            else
            {
                btnSelect.ImageUrl = ResolveUrl(imageBaseUrl + "next.gif");
                btnSelect.ToolTip = ResHelper.GetString("dialogs.list.actions.select");

                // Initialize command
                btnSelect.Attributes["onclick"] = "ColorizeRow('" + GetColorizeID(dr) + "'); SetSelectAction(\"" + argument + ";" + selectUrl + "\"); return false;";
            }
        }

        // Initialize SELECTSUBDOCS button
        ImageButton btnSelectSubDocs = e.Item.FindControl("btnSelectSubDocs") as ImageButton;
        if (btnSelectSubDocs != null)
        {
            if (this.IsDisplayMore && (this.SourceType == MediaSourceEnum.Content))
            {
                int nodeId = ValidationHelper.GetInteger(dr["NodeID"], 0);
                int nodeChildCount = ValidationHelper.GetInteger(dr["NodeChildNodesCount"], 0);

                btnSelectSubDocs.ToolTip = ResHelper.GetString("dialogs.list.actions.showsubdocuments");

                // Check if item is selectable, if not remove select action button
                if (nodeChildCount > 0)
                {
                    // Initialize command
                    btnSelectSubDocs.Attributes["onclick"] = "SetParentAction('" + nodeId + "'); return false;";
                    btnSelectSubDocs.ImageUrl = ResolveUrl(UrlHelper.ImagesDirectory + "/CMSAdminControls/UniGrid/subdocument.gif");
                }
                else
                {
                    btnSelectSubDocs.ImageUrl = ResolveUrl(UrlHelper.ImagesDirectory + "/CMSAdminControls/UniGrid/subdocumentdisabled.gif");
                    btnSelectSubDocs.Attributes["style"] = "cursor:default;";
                    btnSelectSubDocs.Enabled = false;
                }
            }
            else
            {
                btnSelectSubDocs.Visible = false;
            }
        }


        // Initialize VIEW button
        ImageButton btnView = e.Item.FindControl("btnView") as ImageButton;
        if (btnView != null)
        {
            if (!notAttachment)
            {
                btnView.ImageUrl = ResolveUrl(imageBaseUrl + "view.gif");
                btnView.ToolTip = ResHelper.GetString("dialogs.list.actions.view");
                btnView.OnClientClick = "javascript: window.open('" + UrlHelper.ResolveUrl(selectUrl) + "'); return false;";
            }
            else
            {
                btnView.Visible = false;
            }
        }

        // Initialize EDIT button
        ImageButton btnContentEdit = e.Item.FindControl("btnContentEdit") as ImageButton;
        if (btnContentEdit != null)
        {
            btnContentEdit.ToolTip = ResHelper.GetString("general.edit");

            Guid guid = Guid.Empty;
            if (this.SourceType == MediaSourceEnum.MediaLibraries)
            {
                int siteId = ValidationHelper.GetInteger(dr["FileSiteID"], 0);
                guid = ValidationHelper.GetGuid(dr["FileGUID"], Guid.Empty);
                btnContentEdit.ImageUrl = ResolveUrl(UrlHelper.ImagesDirectory + "/CMSAdminControls/Dialogs/edit.gif");
                btnContentEdit.AlternateText = ext + ";MediaFileGUID=" + guid + "&sitename=" + GetSiteName(dr, true);
                btnContentEdit.PreRender += new EventHandler(img_PreRender);
            }
            else if (!notAttachment)
            {
                string nodeid = "";
                if (this.SourceType == MediaSourceEnum.Content)
                {
                    nodeid = "&nodeId=" + dr["NodeID"].ToString();

                    // Get the node workflow
                    this.VersionHistoryID = ValidationHelper.GetInteger(dr["DocumentCheckedOutVersionHistoryID"], 0);
                }

                guid = ValidationHelper.GetGuid(dr["AttachmentGUID"], Guid.Empty);
                btnContentEdit.ImageUrl = ResolveUrl(UrlHelper.ImagesDirectory + "/CMSAdminControls/Dialogs/edit.gif");
                btnContentEdit.AlternateText = ext + ";AttachmentGUID=" + guid + "&sitename=" + GetSiteName(dr, false) + nodeid +
                    ((this.VersionHistoryID > 0) ? "&versionHistoryId=" + this.VersionHistoryID : "");
                btnContentEdit.PreRender += new EventHandler(img_PreRender);
            }
            else
            {
                btnContentEdit.Visible = false;
            }
        }

        // If attachments being displayed show additional actions
        if (this.SourceType == MediaSourceEnum.DocumentAttachments)
        {
            // Initialize EDIT button
            ImageButton btnEdit = e.Item.FindControl("btnEdit") as ImageButton;
            if (btnEdit != null)
            {
                if (!notAttachment)
                {
                    btnEdit.ToolTip = ResHelper.GetString("general.edit");

                    // Get file extension
                    string extension = ValidationHelper.GetString(dr["AttachmentExtension"], "").ToLower();
                    Guid guid = ValidationHelper.GetGuid(dr["AttachmentGUID"], Guid.Empty);

                    // Get the node workflow
                    if ((this.TreeNodeObj != null) && (this.VersionHistoryID == 0))
                    {
                        // Get the node workflow
                        WorkflowManager wm = new WorkflowManager(this.TreeNodeObj.TreeProvider);
                        WorkflowInfo wi = wm.GetNodeWorkflow(this.TreeNodeObj);
                        if (wi != null)
                        {
                            // Ensure the document version
                            VersionManager vm = new VersionManager(this.TreeNodeObj.TreeProvider);
                            this.VersionHistoryID = vm.EnsureVersion(this.TreeNodeObj, this.TreeNodeObj.IsPublished);
                        }
                    }

                    btnEdit.ImageUrl = ResolveUrl(imageBaseUrl + "edit.gif");
                    btnEdit.AlternateText = extension + ";AttachmentGUID=" + guid + "&sitename=" + GetSiteName(dr, false) + "&versionHistoryId=" + this.VersionHistoryID;
                    btnEdit.PreRender += new EventHandler(img_PreRender);
                }
            }

            // Initialize UPDATE button
            CMSAdminControls_Attachments_DirectFileUploader_DirectFileUploader dfuElem = e.Item.FindControl("dfuElem") as CMSAdminControls_Attachments_DirectFileUploader_DirectFileUploader;
            if (dfuElem != null)
            {
                DataRowView drv = (DataRowView)e.Item.DataItem;

                GetAttachmentUpdateControl(ref dfuElem, drv);
            }

            // Initialize DELETE button
            ImageButton btnDelete = e.Item.FindControl("btnDelete") as ImageButton;
            if (btnDelete != null)
            {
                btnDelete.ImageUrl = ResolveUrl(imageBaseUrl + "delete.gif");
                btnDelete.ToolTip = ResHelper.GetString("general.delete");

                // Initialize command
                btnDelete.Attributes["onclick"] = "if(DeleteConfirmation() == false){return false;} SetAction('attachmentdelete','" + dr["AttachmentGUID"] + "'); RaiseHiddenPostBack(); return false;";
            }

            PlaceHolder plcContentEdit = e.Item.FindControl("plcContentEdit") as PlaceHolder;
            plcContentEdit.Visible = false;
        }
        else
        {
            PlaceHolder plcAttachmentActions = e.Item.FindControl("plcAttachmentActions") as PlaceHolder;
            plcAttachmentActions.Visible = false;
        }
    }

    #endregion


    #region "Raise events methods"

    /// <summary>
    /// Fires specific action and returns result provided by the parent control.
    /// </summary>
    /// <param name="dr">Data related to the action.</param>
    private string RaiseOnGetArgumentSet(DataRow dr)
    {
        if (this.GetArgumentSet != null)
        {
            return GetArgumentSet(dr);
        }
        return "";
    }


    /// <summary>
    /// Fires specific action and returns result provided by the parent control.
    /// </summary>
    /// <param name="dr">Data related to the action.</param>
    /// <param name="isPreview">Indicates whether the URL is required for preview item.</param>
    /// <param name="notAttachment">Indicates whether the URL is required for non-attachment item.</param>
    private string RaiseOnGetListItemUrl(DataRow dr, bool isPreview, bool notAttachment)
    {
        if (this.GetListItemUrl != null)
        {
            return GetListItemUrl(dr, isPreview, notAttachment);
        }
        return "";
    }

    /// <summary>
    /// Fires specific action and returns result provided by the parent control.
    /// </summary>
    /// <param name="dr">Data related to the action.</param>
    /// <param name="isPreview">Indicates whether the image is required as part of preview.</param>
    /// <param name="maxSideSize">Maximum size of the preview image. If full-size required parameter gets zero value.</param>
    /// <param name="notAttachment">Indicates whether the URL is required for non-attachment item.</param>
    private string RaiseOnGetTilesThumbsItemUrl(DataRow dr, bool isPreview, int height, int width, int maxSideSize, bool notAttachment)
    {
        if (this.GetTilesThumbsItemUrl != null)
        {
            return GetTilesThumbsItemUrl(dr, isPreview, height, width, maxSideSize, notAttachment);
        }
        return "";
    }

    #endregion
}
