using System;
using System.Collections;
using System.Web.UI.WebControls;
using System.Text;

using CMS.GlobalHelper;
using CMS.UIControls;

public partial class CMSAdminControls_guide : CMSUserControl
{
    private ArrayList mParameters;
    private int mColumns = 1;


    /// <summary>
    /// ArrayList of ArrayLists containig ImageURL, Title, PageURL, Description
    /// </summary>
    public ArrayList Parameters
    {
        get
        {
            return mParameters;
        }
        set
        {
            mParameters = value;
        }
    }


    /// <summary>
    /// Number of columns (default 1).
    /// </summary>
    public int Columns
    {
        get
        {
            return mColumns;
        }
        set
        {
            mColumns = value;
        }
    }


    protected void Page_Load(object sender, EventArgs e)
    {
        if (mParameters != null)
        {
            //Register javascript
            if (!this.Page.ClientScript.IsClientScriptBlockRegistered("detectTreeFrame"))
            {
                this.Page.ClientScript.RegisterClientScriptBlock(typeof(string), "detectTreeFrame", InitScript(), true);
            }

            Panel PanelModules = new Panel();
            Table tbl = new Table();
            tbl.CellSpacing = 10;
            TableRow tr = new TableRow();
            TableCell td;
            int actualRowIndex = 0;
            int relativeWidth = Convert.ToInt32(100/mColumns);
            for (int i = 0; i < mParameters.Count;i++ )
            {
                ArrayList row = (ArrayList) mParameters[i];
                if (row != null)
                {
                    actualRowIndex++;
                    // Initialize Image
                    Image img = new Image();
                    img.ImageUrl = row[0].ToString();
                    img.CssClass = "PageTitleImage";

                    // Initialize Title
                    Label l = new Label();
                    l.Text = " " + ResHelper.LocalizeString(row[1].ToString());

                    // Initialize Hyperlink
                    HyperLink h = new HyperLink();
                    h.Controls.Add(img);
                    h.Controls.Add(l);
                    h.Attributes.Add("onclick", "ShowDesktopContent('" + row[2] + "', '" + row[1] + "');");
                    h.Attributes.Add("href", "#");

                    // Resolve description
                    string description = ResHelper.LocalizeString(row[3].ToString());

                    // Initialize Description
                    Label desc = new Label();
                    desc.Text = "<div>" + description + "</div>";

                    // Initialize wraping pannel
                    Panel p = new Panel();
                    p.Controls.Add(h);
                    p.Controls.Add(desc);

                    // Add style
                    p.CssClass = "Guide";

                    // Add to the table
                    td = new TableCell();
                    // Align all cells to top
                    td.VerticalAlign = VerticalAlign.Top;
                    // Add single description to tablecell
                    td.Controls.Add(p);

                    if (actualRowIndex == mColumns || (i == mParameters.Count-1))
                    {
                        tr.Cells.Add(td);

                        // Ensure right column number for validity
                        if (i == mParameters.Count - 1)
                        {
                            for (int d = 0; d < (mColumns - (mParameters.Count % mColumns)); d++)
                            {
                                tr.Cells.Add(new TableCell());
                            }
                        }
                        // Add to table
                        tbl.Rows.Add(tr);
                        // Reset index counter
                        actualRowIndex = 0;
                        // Create new row
                        tr = new TableRow();
                    }
                    else
                    {
                        // Set relative width
                        td.Attributes.Add("style", "width:" + relativeWidth + "%;");
                        // Add to tablerow
                        tr.Cells.Add(td);
                    }
                }
            }

            // Add single module description to PanelModules
            PanelModules.Controls.Add(tbl);

            // Render whole description
            plcGuide.Controls.Add(PanelModules);
        }
    }


    /// <summary>
    /// Return javascript to detect tree frame.
    /// </summary>
    private static string InitScript()
    {
        StringBuilder builder = new StringBuilder();

        builder.Append("var leftMenuFrame;");
        builder.Append("for(var f = 0;f < parent.frames.length;f++)");
        builder.Append("{");
        builder.Append("if(parent.frames[f].name.indexOf('tree') != -1)");
        builder.Append("leftMenuFrame = parent.frames[f];");
        builder.Append("}");
        builder.Append("var allElems = leftMenuFrame.self.document.getElementsByTagName('*');");
        

        return builder.ToString();
    }
}
