using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using CMS.TreeEngine;
using CMS.GlobalHelper;
using CMS.CMSHelper;
using CMS.WorkflowEngine;
using CMS.SettingsProvider;

using TreeNode = CMS.TreeEngine.TreeNode;
using CMS.UIControls;

public partial class ContentEditMenu : CMSUserControl
{
    #region "Variables"

    private int nodeId = 0;
    private string mAction = null;

    private bool newdocument = false;
    private bool newlink = false;
    private string imagesUrl = null;
    private TreeNode node = null;
    private bool mAllowEdit = true;
    private bool mAllowSave = true;
    private bool mEnablepasiveRefresh = true;
    private string mWorkflowInfo = "";
    private int mMenuItems = 0;

    public event EventHandler LocalSave;

    private bool mShowSave = true;
    private bool mShowCheckOut = true;
    private bool mShowCheckIn = true;
    private bool mShowUndoCheckOut = true;
    private bool mShowApprove = true;
    private bool mShowSubmitToApproval = true;
    private bool mShowReject = true;
    private bool mShowProperties = true;
    private bool mShowSpellCheck = false;
    private bool mShowDelete = false;
    private bool mShowCreateAnother = true;

    #endregion


    #region "Events"

    /// <summary>
    /// On before check in event handler 
    /// </summary>
    public event EventHandler OnBeforeCheckIn;


    /// <summary>
    /// On before check out event handler 
    /// </summary>
    public event EventHandler OnBeforeCheckOut;


    /// <summary>
    /// On before undo check out event handler 
    /// </summary>
    public event EventHandler OnBeforeUndoCheckOut;


    /// <summary>
    /// On before approve event handler 
    /// </summary>
    public event EventHandler OnBeforeApprove;


    /// <summary>
    /// On before reject event handler 
    /// </summary>
    public event EventHandler OnBeforeReject;


    /// <summary>
    /// On before save event handler 
    /// </summary>
    public event EventHandler OnBeforeSave;


    /// <summary>
    /// Raises on before check in event
    /// </summary>
    /// <param name="sender">Sender</param>
    /// <param name="e">Event arguments</param>
    public void RaiseOnBeforeCheckIn(object sender, EventArgs e)
    {
        if (OnBeforeCheckIn != null)
        {
            OnBeforeCheckIn(sender, e);
        }
    }


    /// <summary>
    /// Raises on before check out event
    /// </summary>
    /// <param name="sender">Sender</param>
    /// <param name="e">Event arguments</param>
    public void RaiseOnBeforeCheckOut(object sender, EventArgs e)
    {
        if (OnBeforeCheckOut != null)
        {
            OnBeforeCheckOut(sender, e);
        }
    }


    /// <summary>
    /// Raises on before undo check out event
    /// </summary>
    /// <param name="sender">Sender</param>
    /// <param name="e">Event arguments</param>
    public void RaiseOnBeforeUndoCheckOut(object sender, EventArgs e)
    {
        if (OnBeforeUndoCheckOut != null)
        {
            OnBeforeUndoCheckOut(sender, e);
        }
    }


    /// <summary>
    /// Raises on before approve event
    /// </summary>
    /// <param name="sender">Sender</param>
    /// <param name="e">Event arguments</param>
    public void RaiseOnBeforeApprove(object sender, EventArgs e)
    {
        if (OnBeforeApprove != null)
        {
            OnBeforeApprove(sender, e);
        }
    }


    /// <summary>
    /// Raises on before reject event
    /// </summary>
    /// <param name="sender">Sender</param>
    /// <param name="e">Event arguments</param>
    public void RaiseOnBeforeReject(object sender, EventArgs e)
    {
        if (OnBeforeReject != null)
        {
            OnBeforeReject(sender, e);
        }
    }


    /// <summary>
    /// Raises on before save event
    /// </summary>
    /// <param name="sender">Sender</param>
    /// <param name="e">Event arguments</param>
    public void RaiseOnBeforeSave(object sender, EventArgs e)
    {
        if (OnBeforeSave != null)
        {
            OnBeforeSave(sender, e);
        }
    }

    #endregion


    #region "Button display options"

    /// <summary>
    /// Show the Delete button?
    /// </summary>
    public bool ShowDelete
    {
        get
        {
            return mShowDelete;
        }
        set
        {
            mShowDelete = value;
        }
    }


    /// <summary>
    /// Show the Save button?
    /// </summary>
    public bool ShowSave
    {
        get
        {
            return mShowSave;
        }
        set
        {
            mShowSave = value;
        }
    }


    /// <summary>
    /// Show the Check Out button?
    /// </summary>
    public bool ShowCheckOut
    {
        get
        {
            return mShowCheckOut;
        }
        set
        {
            mShowCheckOut = value;
        }
    }


    /// <summary>
    /// Show the Check In button?
    /// </summary>
    public bool ShowCheckIn
    {
        get
        {
            return mShowCheckIn;
        }
        set
        {
            mShowCheckIn = value;
        }
    }


    /// <summary>
    /// Show the Undo CheckOut button?
    /// </summary>
    public bool ShowUndoCheckOut
    {
        get
        {
            return mShowUndoCheckOut;
        }
        set
        {
            mShowUndoCheckOut = value;
        }
    }


    /// <summary>
    /// Show the Approve button?
    /// </summary>
    public bool ShowApprove
    {
        get
        {
            return mShowApprove;
        }
        set
        {
            mShowApprove = value;
        }
    }


    /// <summary>
    /// Show the Reject button?
    /// </summary>
    public bool ShowReject
    {
        get
        {
            return mShowReject;
        }
        set
        {
            mShowReject = value;
        }
    }


    /// <summary>
    /// Show the Submit To Approval button?
    /// </summary>
    public bool ShowSubmitToApproval
    {
        get
        {
            return mShowSubmitToApproval;
        }
        set
        {
            mShowSubmitToApproval = value;
        }
    }


    /// <summary>
    /// Show the Properties button?
    /// </summary>
    public bool ShowProperties
    {
        get
        {
            return mShowProperties;
        }
        set
        {
            mShowProperties = value;
        }
    }


    /// <summary>
    /// Show spell check button
    /// </summary>
    public bool ShowSpellCheck
    {
        get
        {
            return mShowSpellCheck;
        }
        set
        {
            mShowSpellCheck = value;
        }
    }


    /// <summary>
    /// If true, create another button can be displayed
    /// </summary>
    public bool ShowCreateAnother
    {
        get
        {
            return mShowCreateAnother;
        }
        set
        {
            mShowCreateAnother = value;
        }
    }

    #endregion


    #region "Public properties"

    /// <summary>
    /// Enable passive refresh
    /// </summary>
    public bool EnablePassiveRefresh
    {
        get
        {
            return mEnablepasiveRefresh;
        }
        set
        {
            mEnablepasiveRefresh = value;
        }
    }


    /// <summary>
    /// Returns true if the document editing is allowed
    /// </summary>
    public bool AllowEdit
    {
        get
        {
            return mAllowEdit;
        }
    }


    /// <summary>
    /// Save action allowed
    /// </summary>
    public bool AllowSave
    {
        get
        {
            return mAllowSave;
        }
        set
        {
            mAllowSave = value;
        }
    }


    /// <summary>
    /// If true, the access permissions to the items are checked.
    /// </summary>
    public bool CheckPermissions
    {
        get
        {
            return ValidationHelper.GetBoolean(ViewState["CheckPermissions"], true);
        }
        set
        {
            ViewState["CheckPermissions"] = value;
        }
    }


    /// <summary>
    /// Returns the 
    /// </summary>
    public string WorkflowInfo
    {
        get
        {
            return mWorkflowInfo;
        }
    }


    /// <summary>
    /// Count of the menu items currently displayed
    /// </summary>
    public int MenuItems
    {
        get
        {
            return mMenuItems;
        }
    }


    /// <summary>
    /// Node ID
    /// </summary>
    public int NodeID
    {
        get
        {
            return nodeId;
        }
        set
        {
            nodeId = value;
        }
    }


    /// <summary>
    /// Action
    /// </summary>
    public string Action
    {
        get
        {
            return mAction;
        }
        set
        {
            mAction = value;
        }
    }


    /// <summary>
    /// Render the menu scripts?
    /// </summary>
    public bool RenderScript
    {
        get
        {
            return this.plcScript.Visible;
        }
        set
        {
            this.plcScript.Visible = value;
        }
    }


    /// <summary>
    /// Prefix of the action functions
    /// </summary>
    public string FunctionsPrefix
    {
        get
        {
            return ValidationHelper.GetString(ViewState["FunctionsPrefix"], "");
        }
        set
        {
            ViewState["FunctionsPrefix"] = value;
        }
    }

    #endregion


    protected void Page_Load(object sender, EventArgs e)
    {
        this.btnApprove.Attributes.Add("style", "display: none;");
        this.btnCheckIn.Attributes.Add("style", "display: none;");
        this.btnCheckOut.Attributes.Add("style", "display: none;");
        this.btnReject.Attributes.Add("style", "display: none;");
        this.btnSave.Attributes.Add("style", "display: none;");
        this.btnUndoCheckout.Attributes.Add("style", "display: none;");

        imagesUrl = ResolveUrl(UrlHelper.ImagesDirectory + "/CMSDesk/Content/");

        // Get the document ID
        if (nodeId <= 0)
        {
            nodeId = ValidationHelper.GetInteger(Request.QueryString["nodeid"], 0);
        }

        // Get information whether new document is created
        if (mAction == null)
        {
            mAction = ValidationHelper.GetString(Request.QueryString["action"], "").ToLower();
        }

        ReloadMenu();

        if (this.RenderScript)
        {
            string script = "function " + this.FunctionsPrefix + "LocalSave(nodeId) { " + this.Page.ClientScript.GetPostBackEventReference(this.btnSave, null) + "; } \n" +
                            "function " + this.FunctionsPrefix + "LocalApprove(nodeId) { " + this.Page.ClientScript.GetPostBackEventReference(this.btnApprove, null) + "; } \n" +
                            "function " + this.FunctionsPrefix + "LocalReject(nodeId) { " + this.Page.ClientScript.GetPostBackEventReference(this.btnReject, null) + "; } \n" +
                            "function " + this.FunctionsPrefix + "CheckOut(nodeId) { " + this.Page.ClientScript.GetPostBackEventReference(this.btnCheckOut, null) + "; } \n" +
                            "function " + this.FunctionsPrefix + "LocalCheckIn(nodeId) { " + this.Page.ClientScript.GetPostBackEventReference(this.btnCheckIn, null) + "; } \n" +
                            "function " + this.FunctionsPrefix + "UndoCheckOut(nodeId) { if(!confirm(" + ScriptHelper.GetString(ResHelper.GetString("EditMenu.UndoCheckOutConfirmation")) + ")) return false; " + this.Page.ClientScript.GetPostBackEventReference(this.btnUndoCheckout, null) + "; } \n";

            // Add control frame name
            script += "var controlFrame = 'editview'; \n";

            this.ltlScript.Text += ScriptHelper.GetScript(script);
        }

        this.menuElem.Layout = CMS.skmMenuControl.MenuLayout.Horizontal;
    }


    public void ReloadMenu()
    {
        newlink = (mAction == "newlink");
        newdocument = ((mAction == "new") || newlink);

        CMS.skmMenuControl.MenuItem newItem = null;

        // Create the edit menu
        menuElem.Items.Clear();

        mMenuItems = 0;

        // Save button
        CMS.skmMenuControl.MenuItem saveItem = null;
        if (mShowSave)
        {
            saveItem = new CMS.skmMenuControl.MenuItem();
            saveItem.ToolTip = ResHelper.GetString("EditMenu.Save");
            saveItem.JavascriptCommand = this.FunctionsPrefix + "SaveDocument(" + nodeId.ToString() + ");";
            saveItem.ImageAltText = saveItem.ToolTip;
            saveItem.Text = ResHelper.GetString("general.save");
            saveItem.Image = imagesUrl + "save.gif";
            saveItem.MouseOverImage = saveItem.Image;
            saveItem.MouseOverCssClass = "MenuItemEdit";
            saveItem.CssClass = "MenuItemEdit";
            menuElem.Items.Add(saveItem);
            mMenuItems += 1;
        }

        mWorkflowInfo = "";

        mAllowEdit = true;
        mAllowSave = true;
        bool canApprove = false;
        bool showDelete = false;

        if (!newdocument)
        {
            // Get the document
            TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
            tree.CombineWithDefaultCulture = false;

            //node = tree.SelectSingleNode(nodeId);
            node = DocumentHelper.GetDocument(nodeId, CMSContext.PreferredCultureCode, tree);

            if (node != null)
            {
                bool authorized = true;

                // If the perrmisions should be checked
                if (this.CheckPermissions)
                {
                    // Check read permissions
                    if (CMSContext.CurrentUser.IsAuthorizedPerTreeNode(node, NodePermissionsEnum.Read) == AuthorizationResultEnum.Denied)
                    {
                        RedirectToAccessDenied(String.Format(ResHelper.GetString("cmsdesk.notauthorizedtoreaddocument"), node.NodeAliasPath));
                    }
                    // Check modify permissions
                    else if (CMSContext.CurrentUser.IsAuthorizedPerTreeNode(node, NodePermissionsEnum.Modify) == AuthorizationResultEnum.Denied)
                    {
                        mAllowSave = false;
                        mWorkflowInfo = String.Format(ResHelper.GetString("cmsdesk.notauthorizedtoeditdocument"), node.NodeAliasPath);
                        authorized = false;
                    }
                }

                // If authorized
                if (authorized)
                {
                    WorkflowManager wm = new WorkflowManager(tree);
                    WorkflowInfo wi = wm.GetNodeWorkflow(node);
                    // If workflow not null, process the workflow information to display the items
                    if (wi != null)
                    {
                        // Get current step info
                        WorkflowStepInfo si = wm.GetStepInfo(node);
                        if (si != null)
                        {
                            bool allowApproval = true;
                            canApprove = wm.CanUserApprove(node, CMSContext.CurrentUser);
                            string stepName = si.StepName.ToLower();
                            if (!(canApprove || (stepName == "edit") || (stepName == "published") || (stepName == "archived")))
                            {
                                mAllowSave = false;
                            }

                            // Check-in, Check-out
                            if (VersionManager.UseCheckInCheckOut(CMSContext.CurrentSiteName))
                            {
                                if (node.GetValue("DocumentCheckedOutByUserID") == null)
                                {
                                    mAllowSave = false;
                                    mAllowEdit = false;

                                    // If not checked out, add the check-out button
                                    if (mShowCheckOut && (canApprove || (stepName == "edit") || (stepName == "published") || (stepName == "archived")))
                                    {
                                        newItem = new CMS.skmMenuControl.MenuItem();
                                        newItem.ToolTip = ResHelper.GetString("EditMenu.CheckOut");
                                        newItem.JavascriptCommand = this.FunctionsPrefix + "CheckOut(" + nodeId.ToString() + ");";
                                        newItem.ImageAltText = newItem.ToolTip;
                                        newItem.Text = ResHelper.GetString("EditMenu.IconCheckOut");
                                        newItem.Image = imagesUrl + "checkout.gif";
                                        newItem.MouseOverImage = newItem.Image;
                                        newItem.MouseOverCssClass = "MenuItemEdit";
                                        newItem.CssClass = "MenuItemEdit";
                                        menuElem.Items.Add(newItem);
                                        mMenuItems += 1;
                                    }
                                    // Set workflow info
                                    // If not checked out, add the check-out information
                                    if (canApprove || (stepName == "edit") || (stepName == "published") || (stepName == "archived"))
                                    {
                                        this.mWorkflowInfo += ResHelper.GetString("EditContent.DocumentCheckedIn");
                                    }
                                }
                                else
                                {
                                    // If checked out by current user, add the check-in button
                                    int checkedOutBy = ValidationHelper.GetInteger(node.GetValue("DocumentCheckedOutByUserID"), 0);
                                    if (checkedOutBy == CMSContext.CurrentUser.UserID)
                                    {
                                        if (mShowCheckIn)
                                        {
                                            newItem = new CMS.skmMenuControl.MenuItem();
                                            newItem.ToolTip = ResHelper.GetString("EditMenu.CheckIn");
                                            newItem.JavascriptCommand = this.FunctionsPrefix + "CheckIn(" + nodeId.ToString() + ");";
                                            newItem.ImageAltText = newItem.ToolTip;
                                            newItem.Text = ResHelper.GetString("EditMenu.IconCheckIn");
                                            newItem.Image = imagesUrl + "checkin.gif";
                                            newItem.MouseOverImage = newItem.Image;
                                            newItem.MouseOverCssClass = "MenuItemEdit";
                                            newItem.CssClass = "MenuItemEdit";
                                            menuElem.Items.Add(newItem);
                                            mMenuItems += 1;
                                        }

                                        if (mShowUndoCheckOut)
                                        {
                                            newItem = new CMS.skmMenuControl.MenuItem();
                                            newItem.ToolTip = ResHelper.GetString("EditMenu.UndoCheckout");
                                            newItem.JavascriptCommand = this.FunctionsPrefix + "UndoCheckOut(" + nodeId.ToString() + ");";
                                            newItem.ImageAltText = newItem.ToolTip;
                                            newItem.Text = ResHelper.GetString("EditMenu.IconUndoCheckout");
                                            newItem.Image = imagesUrl + "undocheckout.gif";
                                            newItem.MouseOverImage = newItem.Image;
                                            newItem.MouseOverCssClass = "MenuItemEdit";
                                            newItem.CssClass = "MenuItemEdit";
                                            menuElem.Items.Add(newItem);
                                            mMenuItems += 1;
                                        }
                                        // Set workflow info

                                        this.mWorkflowInfo += ResHelper.GetString("EditContent.DocumentCheckedOut");
                                    }
                                    else
                                    {
                                        mAllowSave = false;
                                        mAllowEdit = false;
                                        // Set workflow info
                                        this.mWorkflowInfo += ResHelper.GetString("EditContent.DocumentCheckedOutByAnother");
                                    }
                                    allowApproval = false;
                                }
                            }

                            // Document approval
                            if (allowApproval)
                            {
                                CMS.skmMenuControl.MenuItem approveItem = null;

                                switch (stepName)
                                {
                                    case "edit":
                                        // When edit step, display submit to approval
                                        if (mShowSubmitToApproval)
                                        {
                                            approveItem = new CMS.skmMenuControl.MenuItem();
                                            approveItem.ToolTip = ResHelper.GetString("EditMenu.SubmitToApproval");
                                            approveItem.JavascriptCommand = this.FunctionsPrefix + "Approve(" + nodeId.ToString() + ");";
                                            approveItem.ImageAltText = approveItem.ToolTip;
                                            approveItem.Text = ResHelper.GetString("EditMenu.IconSubmitToApproval");
                                            approveItem.Image = imagesUrl + "approve.gif";
                                            approveItem.MouseOverImage = approveItem.Image;
                                            approveItem.MouseOverCssClass = "MenuItemEdit";
                                            approveItem.CssClass = "MenuItemEdit";
                                            menuElem.Items.Add(approveItem);
                                            mMenuItems += 1;
                                        }
                                        break;

                                    case "published":
                                    case "archived":
                                        // No workflow options in these steps
                                        break;

                                    default:
                                        // If the user is authorized to perform the step, display the approve and reject buttons
                                        if (canApprove)
                                        {
                                            if (mShowApprove)
                                            {
                                                approveItem = new CMS.skmMenuControl.MenuItem();
                                                approveItem.ToolTip = ResHelper.GetString("EditMenu.Approve");
                                                approveItem.JavascriptCommand = this.FunctionsPrefix + "Approve(" + nodeId.ToString() + ");";
                                                approveItem.ImageAltText = approveItem.ToolTip;
                                                approveItem.Text = ResHelper.GetString("general.approve");
                                                approveItem.Image = imagesUrl + "approve.gif";
                                                approveItem.MouseOverImage = approveItem.Image;
                                                approveItem.MouseOverCssClass = "MenuItemEdit";
                                                approveItem.CssClass = "MenuItemEdit";
                                                menuElem.Items.Add(approveItem);
                                                mMenuItems += 1;
                                            }

                                            if (mShowReject)
                                            {
                                                newItem = new CMS.skmMenuControl.MenuItem();
                                                newItem.ToolTip = ResHelper.GetString("EditMenu.Reject");
                                                newItem.JavascriptCommand = this.FunctionsPrefix + "Reject(" + nodeId.ToString() + ");";
                                                newItem.ImageAltText = newItem.ToolTip;
                                                newItem.Text = ResHelper.GetString("general.reject");
                                                newItem.Image = imagesUrl + "reject.gif";
                                                newItem.MouseOverImage = newItem.Image;
                                                newItem.MouseOverCssClass = "MenuItemEdit";
                                                newItem.CssClass = "MenuItemEdit";
                                                menuElem.Items.Add(newItem);
                                                mMenuItems += 1;
                                            }
                                        }
                                        break;
                                }

                                if (approveItem != null)
                                {
                                    // Get next step info
                                    WorkflowStepInfo nsi = wm.GetNextStepInfo(node);
                                    if (nsi.StepName.ToLower() == "published")
                                    {
                                        approveItem.ToolTip = ResHelper.GetString("EditMenu.Publish");
                                        approveItem.Text = ResHelper.GetString("EditMenu.IconPublish");
                                    }
                                }

                                // Set workflow info
                                if (!(canApprove || (stepName == "edit") || (stepName == "published") || (stepName == "archived")))
                                {
                                    if (!string.IsNullOrEmpty(this.mWorkflowInfo))
                                    {
                                        this.mWorkflowInfo += "<br />";
                                    }
                                    this.mWorkflowInfo += ResHelper.GetString("EditContent.NotAuthorizedToApprove");
                                }
                                if (!string.IsNullOrEmpty(this.mWorkflowInfo))
                                {
                                    this.mWorkflowInfo += "<br />";
                                }
                                this.mWorkflowInfo += String.Format(ResHelper.GetString("EditContent.CurrentStepInfo"), si.StepDisplayName);
                            }
                        }
                    }
                }

                if (mShowProperties)
                {
                    newItem = new CMS.skmMenuControl.MenuItem();
                    newItem.ToolTip = ResHelper.GetString("EditMenu.Properties");
                    newItem.JavascriptCommand = this.FunctionsPrefix + "Properties(" + nodeId.ToString() + ");";
                    newItem.ImageAltText = newItem.ToolTip;
                    newItem.Text = ResHelper.GetString("EditMenu.IconProperties");
                    newItem.Image = imagesUrl + "properties.gif";
                    newItem.MouseOverImage = newItem.Image;
                    newItem.MouseOverCssClass = "MenuItemEdit";
                    newItem.CssClass = "MenuItemEdit";
                    menuElem.Items.Add(newItem);
                    mMenuItems += 1;
                }

                if (mShowDelete)
                {
                    showDelete = true;
                }
            }

            // If not allowed to save, disable the save item
            if (!mAllowSave && mShowSave)
            {
                saveItem.JavascriptCommand = "";
                saveItem.MouseOverCssClass = "MenuItemEditDisabled";
                saveItem.CssClass = "MenuItemEditDisabled";
                saveItem.Image = imagesUrl + "savedisabled.gif";
                saveItem.MouseOverImage = saveItem.Image;
            }
        }
        else if (mAllowSave && mShowCreateAnother)
        {
            newItem = new CMS.skmMenuControl.MenuItem();
            newItem.ToolTip = ResHelper.GetString("EditMenu.SaveAndAnother");
            newItem.JavascriptCommand = this.FunctionsPrefix + "SaveDocument(" + nodeId.ToString() + ", true);";
            newItem.ImageAltText = newItem.ToolTip;
            newItem.Text = ResHelper.GetString("editmenu.iconsaveandanother");
            newItem.Image = imagesUrl + "saveandanother.gif";
            newItem.MouseOverImage = newItem.Image;
            newItem.MouseOverCssClass = "MenuItemEdit";
            newItem.CssClass = "MenuItemEdit";
            menuElem.Items.Add(newItem);
            mMenuItems += 1;
        }

        if (mAllowSave && mShowSave && mShowSpellCheck && !newlink)
        {
            newItem = new CMS.skmMenuControl.MenuItem();
            newItem.ToolTip = ResHelper.GetString("EditMenu.SpellCheck");
            newItem.JavascriptCommand = this.FunctionsPrefix + "SpellCheck();";
            newItem.ImageAltText = newItem.ToolTip;
            newItem.Text = ResHelper.GetString("EditMenu.IconSpellCheck");
            newItem.Image = imagesUrl + "spellcheck.gif";
            newItem.MouseOverImage = newItem.Image;
            newItem.MouseOverCssClass = "MenuItemEdit";
            newItem.CssClass = "MenuItemEdit";
            menuElem.Items.Add(newItem);
            mMenuItems += 1;
        }

        if (showDelete)
        {
            newItem = new CMS.skmMenuControl.MenuItem();
            newItem.ToolTip = ResHelper.GetString("EditMenu.Delete");
            newItem.JavascriptCommand = this.FunctionsPrefix + "Delete(" + nodeId.ToString() + ");";
            newItem.ImageAltText = newItem.ToolTip;
            newItem.Text = ResHelper.GetString("general.delete");
            newItem.Image = imagesUrl + "delete.gif";
            newItem.MouseOverImage = newItem.Image;
            newItem.MouseOverCssClass = "MenuItemEdit";
            newItem.CssClass = "MenuItemEdit";
            menuElem.Items.Add(newItem);
            mMenuItems += 1;
        }
    }


    protected void btnApprove_Click(object sender, EventArgs e)
    {
        if (node != null)
        {
            // Raise event
            RaiseOnBeforeApprove(sender, e);

            // Approve the document - Go to next workflow step
            TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
            WorkflowManager wm = new WorkflowManager(tree);

            // Get original step
            WorkflowStepInfo originalStep = wm.GetStepInfo(node);

            // Approve the document
            WorkflowStepInfo nextStep = wm.MoveToNextStep(node, "");

            if (nextStep != null)
            {
                // Send workflow e-mails
                if (SettingsKeyProvider.GetBoolValue(CMSContext.CurrentSiteName + ".CMSSendWorkflowEmails"))
                {
                    if (nextStep.StepName.ToLower() == "published")
                    {
                        // Publish e-mails
                        wm.SendWorkflowEmails(node, CMSContext.CurrentUser, originalStep, nextStep, WorkflowActionEnum.Published, "");
                    }
                    else
                    {
                        // Approve e-mails
                        wm.SendWorkflowEmails(node, CMSContext.CurrentUser, originalStep, nextStep, WorkflowActionEnum.Approved, "");
                    }
                }
            }
        }

        ReloadMenu();
        if (this.EnablePassiveRefresh)
        {
            this.ltlScript.Text += ScriptHelper.GetScript("if(PassiveRefresh){ PassiveRefresh(" + nodeId + "); }");
        }
    }


    protected void btnReject_Click(object sender, EventArgs e)
    {
        // Reject the document - Go to previous workflow step
        if (node != null)
        {
            // Raise event
            RaiseOnBeforeReject(sender, e);

            TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
            WorkflowManager wm = new WorkflowManager(tree);

            // Get original step
            WorkflowStepInfo originalStep = wm.GetStepInfo(node);

            // Reject the document
            WorkflowStepInfo previousStep = wm.MoveToPreviousStep(node, "");

            // Send workflow e-mails
            if (SettingsKeyProvider.GetBoolValue(CMSContext.CurrentSiteName + ".CMSSendWorkflowEmails"))
            {
                wm.SendWorkflowEmails(node, CMSContext.CurrentUser, originalStep, previousStep, WorkflowActionEnum.Rejected, "");
            }
        }
        ReloadMenu();
        if (this.EnablePassiveRefresh)
        {
            this.ltlScript.Text += ScriptHelper.GetScript("if(PassiveRefresh){ PassiveRefresh(" + nodeId + "); }");
        }
    }


    protected void btnCheckIn_Click(object sender, EventArgs e)
    {
        // Check in the document
        if (node != null)
        {
            // Raise event
            RaiseOnBeforeCheckIn(sender, e);

            TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
            VersionManager verMan = new VersionManager(tree);

            // Check in the document        
            verMan.CheckIn(node, null, null);
        }

        ReloadMenu();

        if (this.EnablePassiveRefresh)
        {
            this.ltlScript.Text += ScriptHelper.GetScript("if(PassiveRefresh){ PassiveRefresh(" + nodeId + "); }");
        }
    }


    protected void btnCheckOut_Click(object sender, EventArgs e)
    {
        // Check out the document
        if (node != null)
        {
            // Raise event
            RaiseOnBeforeCheckOut(sender, e);

            TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
            VersionManager verMan = new VersionManager(tree);

            verMan.EnsureVersion(node, node.IsPublished);

            bool wasArchived = (node.WorkflowStepName.ToLower() == "archived");

            // Check out the document
            verMan.CheckOut(node);

            if (wasArchived)
            {
                this.ltlScript.Text += ScriptHelper.GetScript("if (window.RefreshTree) { RefreshTree(" + node.NodeParentID + ", " + node.NodeID + "); }");
            }
        }

        ReloadMenu();

        if (this.EnablePassiveRefresh)
        {
            this.ltlScript.Text += ScriptHelper.GetScript("if (window.PassiveRefresh) { PassiveRefresh(" + nodeId + "); }");
        }
    }


    protected void btnUndoCheckout_Click(object sender, EventArgs e)
    {
        // Check out the document
        if (node != null)
        {
            // Raise event
            RaiseOnBeforeUndoCheckOut(sender, e);

            TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
            VersionManager verMan = new VersionManager(tree);

            // Undo check out
            verMan.UndoCheckOut(node);

            ReloadMenu();

            if (this.EnablePassiveRefresh)
            {
                this.ltlScript.Text += ScriptHelper.GetScript("if(PassiveRefresh){ PassiveRefresh(" + nodeId + "); } if(RefreshTree){ RefreshTree(" + node.NodeParentID + ", " + nodeId + "); }");
            }
        }
    }


    protected void btnSave_Click(object sender, EventArgs e)
    {
        if (LocalSave != null)
        {
            // Raise event
            RaiseOnBeforeSave(sender, e);

            LocalSave(sender, e);
        }
    }
}
