using System;
using System.Data;
using System.Web.UI.WebControls;
using System.Collections;

using CMS.GlobalHelper;
using CMS.TreeEngine;
using CMS.SiteProvider;
using CMS.CMSHelper;
using CMS.UIControls;
using CMS.WorkflowEngine;
using CMS.DataEngine;
using CMS.SettingsProvider;

using TreeNode = CMS.TreeEngine.TreeNode;

public partial class CMSDesk_Content_listing : CMSDeskPage
{
    #region "Protected variables"

    protected string checkedNodeIDs = string.Empty;
    protected ArrayList mFlagsControls = null;
    protected DataSet mSiteCultures = null;
    protected string mDefaultSiteCulture = null;
    protected int nodeId = 0;

    #endregion


    #region "Private properties"

    /// <summary>
    /// Default culture of the site
    /// </summary>
    public string DefaultSiteCulture
    {
        get
        {
            if (mDefaultSiteCulture == null)
            {
                mDefaultSiteCulture = CultureHelper.GetDefaultCulture(CMSContext.CurrentSiteName);
            }
            return mDefaultSiteCulture;
        }
    }


    /// <summary>
    /// Hashtable with document flags controls
    /// </summary>
    private ArrayList FlagsControls
    {
        get
        {
            if (mFlagsControls == null)
            {
                mFlagsControls = new ArrayList();
            }
            return mFlagsControls;
        }
    }


    /// <summary>
    /// Site cultures
    /// </summary>
    private DataSet SiteCultures
    {
        get
        {
            if (mSiteCultures == null)
            {
                mSiteCultures = CultureInfoProvider.GetSiteCultures(CMSContext.CurrentSiteName).Copy();
                if (!DataHelper.DataSourceIsEmpty(mSiteCultures))
                {
                    DataTable cultureTable = mSiteCultures.Tables[0];
                    DataRow[] defaultCultureRow = cultureTable.Select("CultureCode='" + DefaultSiteCulture + "'");

                    // Ensure default culture to be first
                    DataRow dr = cultureTable.NewRow();
                    if (defaultCultureRow.Length > 0)
                    {
                        dr.ItemArray = defaultCultureRow[0].ItemArray;
                        cultureTable.Rows.InsertAt(dr, 0);
                        cultureTable.Rows.Remove(defaultCultureRow[0]);
                    }
                }

            }
            return mSiteCultures;
        }
    }


    /// <summary>
    /// Current page index
    /// </summary>
    private int PageIndex
    {
        get
        {
            return ValidationHelper.GetInteger(ViewState["PageIndex"], 0);
        }
        set
        {
            ViewState["PageIndex"] = value;
        }
    }


    /// <summary>
    /// Where condition is depend on filter value
    /// </summary>
    private string WhereCondition
    {
        get
        {
            if (ViewState["WhereCondition"] != null)
            {
                return ViewState["WhereCondition"].ToString();
            }

            return null;
        }
        set
        {
            ViewState["WhereCondition"] = value;
        }
    }

    #endregion


    #region "Page events"

    /// <summary>
    /// Page load.
    /// </summary>
    protected void Page_Load(object sender, EventArgs e)
    {
        InitDropdowLists();

        // Setup page title text and image
        CurrentMaster.Title.TitleText = ResHelper.GetString("Content.ListingTitle");
        CurrentMaster.Title.TitleImage = UrlHelper.ImagesDirectory + "/CMSDesk/Content/listing.gif";

        CurrentMaster.Title.HelpName = "helpTopic";
        CurrentMaster.Title.HelpTopicName = "list_tab";

        string[,] actions = new string[1, 6];
        actions[0, 0] = HeaderActions.TYPE_HYPERLINK;
        actions[0, 1] = ResHelper.GetString("Listing.ParentDirectory");
        actions[0, 2] = null;
        actions[0, 3] = null;
        actions[0, 4] = null;
        actions[0, 5] = UrlHelper.ImagesDirectory + "/CMSDesk/Content/parent.gif";
        CurrentMaster.HeaderActions.Actions = actions;

        gridDocuments.GridView.PageSize = 20;
        gridDocuments.GridView.GridLines = GridLines.Horizontal;
        gridDocuments.OnExternalDataBound += gridDocuments_OnExternalDataBound;
        gridDocuments.OnBeforePageIndexChanging += gridDocuments_OnBeforePageIndexChanging;
        gridDocuments.OnShowButtonClick += gridDocuments_OnShowButtonClick;

        // Current Node ID
        nodeId = QueryHelper.GetInteger("nodeid", 0);

        if (!RequestHelper.IsPostBack() || !ValidationHelper.GetString(Request.Params["__EVENTTARGET"], "").StartsWith("gridDocuments") || ValidationHelper.GetString(Request.Params["__EVENTARGUMENT"], string.Empty).StartsWith("Sort"))
        {
            ReloadData();
        }
    }


    /// <summary>
    /// OnPreRender.
    /// </summary>
    protected override void OnPreRender(EventArgs e)
    {
        // Hide column with languages if only one culture is assigned to the site
        if (DataHelper.DataSourceIsEmpty(SiteCultures) || (SiteCultures.Tables[0].Rows.Count <= 1))
        {
            DataControlField languagesColumn = gridDocuments.GridView.Columns[6];
            languagesColumn.Visible = false;
            plcLang.Visible = false;
        }
        else
        {
            if (FlagsControls.Count != 0)
            {
                // Get all document node IDs
                string nodeIds = null;
                foreach (DocumentFlagsControl ucDocFlags in FlagsControls)
                {
                    nodeIds += ucDocFlags.NodeID + ",";
                }

                if (nodeIds != null)
                {
                    nodeIds = nodeIds.TrimEnd(',');
                }

                // Get all documents
                TreeProvider tree = new TreeProvider();
                tree.SelectQueryName = "SelectVersions";
                DataSet dsDocs = tree.SelectNodes(CMSContext.CurrentSiteName, "/%", TreeProvider.ALL_CULTURES, false, null, "NodeID IN (" + nodeIds + ")", null, -1, false);
                GroupedDataSource gDSDocs = new GroupedDataSource(dsDocs, "NodeID");

                // Initialize the document flags controls
                foreach (DocumentFlagsControl ucDocFlags in FlagsControls)
                {
                    ucDocFlags.DataSource = gDSDocs;
                    ucDocFlags.ReloadData();
                }
            }
        }

        if (DataHelper.DataSourceIsEmpty(gridDocuments.DataSource))
        {
            lblInfo.Text = ResHelper.GetString("content.nochilddocumentsfound");
            lblInfo.Visible = true;
        }
        else
        {
            pnlFooter.Visible = true;
        }

        base.OnPreRender(e);
    }

    #endregion


    #region "Control events"

    /// <summary>
    /// Checked changed.
    /// </summary>
    protected void chkSelect_CheckedChanged(object sender, EventArgs e)
    {
        CheckBox chk = (CheckBox)sender;
        if (chk.Checked)
        {
            checkedNodeIDs += ";" + chk.ID.Substring(3);
        }
    }


    /// <summary>
    /// Delete click handler.
    /// </summary>
    protected void btnDeleteSelected_Click(object sender, EventArgs e)
    {
        string ids = checkedNodeIDs.Trim(';');
        if (ids != string.Empty)
        {
            UrlHelper.Redirect("delete.aspx?nodeid=" + ids);
        }
    }

    #endregion


    #region "Grid events"

    /// <summary>
    /// OnShow button click handler, cleare where condition.
    /// </summary>
    protected void gridDocuments_OnShowButtonClick(object sender, EventArgs e)
    {
        ReloadData();
    }


    /// <summary>
    /// OnBefore page index changing.
    /// </summary>
    protected void gridDocuments_OnBeforePageIndexChanging(object sender, GridViewPageEventArgs e)
    {
        PageIndex = e.NewPageIndex - (e.NewPageIndex % 10);

        ReloadData();
        gridDocuments.ReloadData();
    }


    /// <summary>
    /// External data binding handler.
    /// </summary>
    protected object gridDocuments_OnExternalDataBound(object sender, string sourceName, object parameter)
    {
        int nodeId = 0;

        switch (sourceName.ToLower())
        {
            case "published":
                bool published = ValidationHelper.GetBoolean(parameter, true);
                if (published)
                {
                    return "<span class=\"DocumentPublishedYes\">" + ResHelper.GetString("General.Yes") + "</span>";
                }
                else
                {
                    return "<span class=\"DocumentPublishedNo\">" + ResHelper.GetString("General.No") + "</span>";
                }

            case "select":
                CheckBox chkSelect = new CheckBox();
                chkSelect.CheckedChanged += chkSelect_CheckedChanged;
                chkSelect.ID = "chk" + parameter;
                return chkSelect;

            case "versionnumber":
                if (parameter == DBNull.Value)
                {
                    parameter = "-";
                }
                return parameter;

            case "documentname":
                DataRowView data = (DataRowView)parameter;

                string name = ValidationHelper.GetString(data["DocumentName"], string.Empty);
                string culture = ValidationHelper.GetString(data["DocumentCulture"], string.Empty);
                string cultureString = null;
                nodeId = ValidationHelper.GetInteger(data["NodeID"], 0);
                int nodeParentId = ValidationHelper.GetInteger(data["NodeParentID"], 0);

                // Default culture
                if (culture.ToLower() != CMSContext.PreferredCultureCode.ToLower())
                {
                    cultureString = " (" + culture + ")";
                }

                string result = "<a href=\"javascript: SelectItem(" + nodeId + ", " + nodeParentId + ");\">" + HTMLHelper.HTMLEncode(name) + cultureString + "</a>";
                bool isLink = (data["NodeLinkedNodeID"] != DBNull.Value);
                if (isLink)
                {
                    result += LINK_MARK;
                }

                return result;

            case "documentculture":
                // Dynamically load document flags control
                DocumentFlagsControl ucDocFlags = Page.LoadControl("~/CMSAdminControls/UI/DocumentFlags.ascx") as DocumentFlagsControl;

                // Set document flags properties
                if (ucDocFlags != null)
                {
                    // Get node ID
                    nodeId = ValidationHelper.GetInteger(parameter, 0);

                    ucDocFlags.ID = "docFlags" + nodeId;
                    ucDocFlags.SiteCultures = SiteCultures;
                    ucDocFlags.NodeID = nodeId;
                    ucDocFlags.StopProcessing = true;

                    // Keep the control for later usage
                    FlagsControls.Add(ucDocFlags);
                    return ucDocFlags;
                }
                break;
        }
        return parameter;
    }

    #endregion


    #region "Other methods"

    /// <summary>
    /// Reload data.
    /// </summary>
    public DataSet ReloadData()
    {
        DataSet toReturn = null;

        if (nodeId > 0)
        {
            TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
            TreeNode node = tree.SelectSingleNode(nodeId, TreeProvider.ALL_CULTURES);
            if (node != null)
            {
                SiteInfo si = SiteInfoProvider.GetSiteInfo(node.NodeSiteID);
                if (si != null)
                {
                    string wherecond = GetWhereCondition();

                    toReturn = DocumentHelper.GetDocuments(si.SiteName, node.NodeAliasPath.TrimEnd('/') + "/%",
                                                TreeProvider.ALL_CULTURES, true, null,
                                                "NodeAliasPath <> '/' " + wherecond,
                                                "NodeOrder ASC, NodeName ASC, NodeAlias ASC", 1, false, tree);


                    gridDocuments.DataSource = toReturn;

                    if (DataHelper.DataSourceIsEmpty(toReturn))
                    {
                        pnlFooter.Visible = false;
                    }
                }

                if ((node.NodeParentID > 0) && (CMSContext.CurrentUser.UserStartingAliasPath != node.NodeAliasPath))
                {
                    CurrentMaster.HeaderActions.Actions[0, 3] = "javascript:SelectItem(" + node.NodeParentID + ");";
                }
                else
                {
                    CurrentMaster.HeaderActions.Visible = false;
                    CurrentMaster.PanelBody.FindControl("pnlActions").Visible = false;
                }
            }
        }

        return toReturn;
    }


    private string GetWhereCondition()
    {
        string where = null;

        if (txtName.Text.Trim() != "")
        {
            switch (drpOperator.SelectedValue)
            {
                case "NOT LIKE":
                case "LIKE":
                    where = " AND DocumentName " + drpOperator.SelectedValue.Replace("'", "''") + "'%" + txtName.Text.Trim().Replace("'", "''") + "%'";
                    break;

                default:
                    where = " AND DocumentName " + drpOperator.SelectedValue.Replace("'", "''") + "'" + txtName.Text.Trim().Replace("'", "''") + "'";
                    break;
            }
        }

        if (drpCulture.SelectedValue != "##ANY##")
        {
            switch (drpCulture.SelectedValue)
            {
                case "##ALL##":
                    where += " AND ((SELECT COUNT(*) FROM View_CMS_Tree_Joined AS TreeView WHERE TreeView.NodeID = View_CMS_Tree_Joined_Versions.NodeID) " + drpLanguage.SelectedValue.Replace("'", "''") + " " + SiteCultures.Tables[0].Rows.Count + ")";
                    break;

                default:
                    string oper = (drpLanguage.SelectedValue == "<>") ? "NOT" : "";
                    where += " AND NodeID " + oper + " IN (SELECT NodeID FROM View_CMS_Tree_Joined AS TreeView WHERE TreeView.NodeID = NodeID AND DocumentCulture = '" + drpCulture.SelectedValue.Replace("'", "''") + "')";
                    break;
            }
        }
        else if (drpLanguage.SelectedValue == "<>")
        {
            where = " AND " + SqlHelperClass.NO_DATA_WHERE;
        }

        return where;
    }



    private void InitDropdowLists()
    {
        // Init cultures
        if (drpCulture.Items.Count == 0)
        {
            drpCulture.DataSource = SiteCultures;
            drpCulture.DataTextField = "CultureName";
            drpCulture.DataValueField = "CultureCode";
            drpCulture.DataBind();

            drpCulture.Items.Insert(0, new ListItem(ResHelper.GetString("transman.allcultures"), "##ALL##"));
            drpCulture.Items.Insert(0, new ListItem(ResHelper.GetString("transman.anyculture"), "##ANY##"));
        }

        // Init operands
        if (drpLanguage.Items.Count == 0)
        {
            drpLanguage.Items.Add(new ListItem(ResHelper.GetString("transman.is"), "="));
            drpLanguage.Items.Add(new ListItem(ResHelper.GetString("transman.isnot"), "<>"));
        }

        // Init operands
        if (drpOperator.Items.Count == 0)
        {
            drpOperator.Items.Add(new ListItem("LIKE", "LIKE"));
            drpOperator.Items.Add(new ListItem("NOT LIKE", "NOT LIKE"));
            drpOperator.Items.Add(new ListItem("=", "="));
            drpOperator.Items.Add(new ListItem("<>", "<>"));
        }
    }

    #endregion
}
