using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using CMS.UIControls;
using CMS.GlobalHelper;
using CMS.TreeEngine;
using CMS.CMSHelper;
using CMS.LicenseProvider;
using CMS.SettingsProvider;

using TreeNode = CMS.TreeEngine.TreeNode;
using CMS.SiteProvider;

public partial class CMSModules_Blogs_Controls_NewBlog : CMSUserControl
{
    #region "Variables"

    string mBlogParentPath = "";
    string mBlogSideColumnText = "";
    Guid mBlogTeaser = Guid.Empty;
    int mBlogOpenCommentsFor = -1; // blog is opened "Always" by default
    string mBlogSendCommentsToEmail = "";
    bool mBlogAllowAnonymousComments = true;
    bool mBlogUseCAPTCHAForComments = false;
    bool mBlogModerateComments = false;
    string mBlogModerators = "";
    bool mRedirectToNewBlog = false;

    #endregion


    #region "Public properties"

    /// <summary>
    /// Path in the content tree where new blog should be created.
    /// </summary>
    public string BlogParentPath
    {
        get
        {
        	 return mBlogParentPath; 
        }
        set
        {
        	 mBlogParentPath = value; 
        }
    }


    /// <summary>
    /// Indicates if user should be redirected to the blog after the blog it is created.
    /// </summary>
    public bool RedirectToNewBlog
    {
        get
        {
        	 return mRedirectToNewBlog; 
        }
        set
        {
        	 mRedirectToNewBlog = value; 
        }
    }


    /// <summary>
    /// Blog side columnt text.
    /// </summary>
    public string BlogSideColumnText
    {
        get
        {
        	 return mBlogSideColumnText; 
        }
        set
        {
        	 mBlogSideColumnText = value; 
        }
    }


    /// <summary>
    /// Blog teaser.
    /// </summary>
    public Guid BlogTeaser
    {
        get
        {
        	 return mBlogTeaser; 
        }
        set
        {
        	 mBlogTeaser = value; 
        }
    }


    /// <summary>
    /// Email address where new comments should be sent.
    /// </summary>
    public string BlogSendCommentsToEmail
    {
        get
        {
        	 return mBlogSendCommentsToEmail; 
        }
        set
        {
        	 mBlogSendCommentsToEmail = value; 
        }
    }


    /// <summary>
    /// Indicates if blog comments are opened (0 - not opened, -1 - always opened, X - number of days the comments are opened after the post is published)
    /// </summary>
    public int BlogOpenCommentsFor
    {
        get
        {
        	 return mBlogOpenCommentsFor; 
        }
        set
        {
        	 mBlogOpenCommentsFor = value; 
        }
    }


    /// <summary>
    /// Indicates if new comments requir to be moderated before publishing.
    /// </summary>
    public bool BlogModerateComments
    {
        get
        {
        	 return mBlogModerateComments; 
        }
        set
        {
        	 mBlogModerateComments = value; 
        }
    }


    /// <summary>
    /// Indicates if security control should be used when inserting new comment.
    /// </summary>
    public bool BlogUseCAPTCHAForComments
    {
        get
        {
        	 return mBlogUseCAPTCHAForComments; 
        }
        set
        {
        	 mBlogUseCAPTCHAForComments = value; 
        }
    }


    /// <summary>
    /// Indicates anonymous users can insert comments.
    /// </summary>
    public bool BlogAllowAnonymousComments
    {
        get
        {
        	 return mBlogAllowAnonymousComments; 
        }
        set
        {
        	 mBlogAllowAnonymousComments = value; 
        }
    }


    /// <summary>
    /// Users which are allowed to moderate blog comments. Format [username1];[username2];...
    /// </summary>
    public string BlogModerators
    {
        get
        {
        	 return mBlogModerators; 
        }
        set
        {
        	 mBlogModerators = value; 
        }
    }

    #endregion


    protected void Page_Load(object sender, EventArgs e)
    {
        if (this.StopProcessing)
        {
            this.Visible = false;
        }
        else
        {
            // Initialize controls
            lblName.Text = ResHelper.GetString("Blogs.NewBlog.Name");
            lblDescription.Text = ResHelper.GetString("Blogs.NewBlog.Description");
            btnOk.Text = ResHelper.GetString("General.OK");
            rfvName.ErrorMessage = ResHelper.GetString("Blogs.NewBlog.NameEmpty");
            btnOk.Click += new EventHandler(btnOk_Click);
        }
    }


    void btnOk_Click(object sender, EventArgs e)
    {
        // Validate all required data for new blog
        string errorMessage = ValidateData();

        if (!LicenseHelper.LicenseVersionCheck(UrlHelper.GetCurrentDomain(), FeatureEnum.Blogs, VersionActionEnum.Insert))
        {
            errorMessage = ResHelper.GetString("cmsdesk.bloglicenselimits");
        }

        if (errorMessage == "")
        {
            // Get parent node for new blog
            TreeProvider tree = new  TreeProvider(CMSContext.CurrentUser);
            TreeNode parent = tree.SelectSingleNode(CMSContext.CurrentSiteName, this.BlogParentPath.TrimEnd('%'), TreeProvider.ALL_CULTURES);
            if (parent != null)
            {
                // Initialize and create new blog node
                TreeNode blogNode = new CMS.TreeEngine.TreeNode("cms.blog", tree);
                blogNode.SetValue("BlogName", txtName.Text.Trim());
                blogNode.SetValue("BlogDescription", txtDescription.Text.Trim());
                blogNode.SetValue("BlogAllowAnonymousComments", this.BlogAllowAnonymousComments );
                blogNode.SetValue("BlogModerateComments", this.BlogModerateComments );
                blogNode.SetValue("BlogOpenCommentsFor", this.BlogOpenCommentsFor );
                blogNode.SetValue("BlogSendCommentsToEmail", this.BlogSendCommentsToEmail);
                blogNode.SetValue("BlogSideColumnText", this.BlogSideColumnText );
                blogNode.SetValue("BlogUseCAPTCHAForComments", this.BlogUseCAPTCHAForComments);
                blogNode.SetValue("BlogModerators", this.BlogModerators);
                if (this.BlogTeaser == Guid.Empty)
                {
                    blogNode.SetValue("BlogTeaser", null);
                }
                else
                {
                    blogNode.SetValue("BlogTeaser", this.BlogTeaser);
                }

                blogNode.SetValue("NodeOwner", CMSContext.CurrentUser.UserID);
                
                // Trackbacks are denied by default
                blogNode.SetValue("BlogEnableTrackbacks", false);
                
                blogNode.DocumentName = txtName.Text.Trim();
                blogNode.DocumentCulture = CMSContext.CurrentUser.PreferredCultureCode;
                blogNode.Insert(parent.NodeID);

                // Update search index for new node
                if ((blogNode.PublishedVersionExists) && (SearchIndexInfoProvider.SearchEnabled))
                {
                    SearchTaskInfoProvider.CreateTask(SearchTaskTypeEnum.Update, PredefinedObjectType.DOCUMENT, SearchHelper.ID_FIELD, blogNode.GetSearchID());
                }
             
                if (this.RedirectToNewBlog)
                {
                    // Redirect to the new blog
                    UrlHelper.Redirect(CMSContext.GetUrl(blogNode.NodeAliasPath));
                }
                else
                {
                    // Display info message
                    lblInfo.Visible = true;
                    lblInfo.Text = ResHelper.GetString("General.ChangesSaved");
                }
            }
            else
            {
                // Parent node was not found
                errorMessage = ResHelper.GetString("Blogs.NewBlog.PathNotFound");
            }
        }
        
        if (errorMessage != "")
        {
            // Display error message
            lblError.Visible = true;
            lblError.Text = errorMessage;
        }
    }


    /// <summary>
    /// Validates form data and returns error message if some error occurs
    /// </summary>
    private string ValidateData()
    {       
        if (txtName.Text.Trim() == "")
        {
            // Blog name is empty
            return rfvName.ErrorMessage;
        }
        if (this.BlogParentPath.TrimEnd('%') == "")
        {
            // Path where blog should be created is empty
            return ResHelper.GetString("Blogs.NewBlog.PathEmpty");
        }
        if (CMSContext.CurrentUser.IsPublic())
        {
            // Anonymous user is not allowed to create blog
            return ResHelper.GetString("Blogs.NewBlog.AnonymousUser");
        }
        return "";
    }
}
