using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using CMS.GlobalHelper;
using CMS.SiteProvider;
using CMS.PortalEngine;
using CMS.TreeEngine;
using CMS.CMSHelper;
using CMS.UIControls;

using TreeNode = CMS.TreeEngine.TreeNode;
using CMS.SettingsProvider;

public partial class CMSImportExport_SelectMasterTemplate : CMSUserControl
{
    /// <summary>
    /// Template ID
    /// </summary>
    public int MasterTemplateId
    {
        get 
        { 
            return ucSelector.SelectedId; 
        }
    }


    /// <summary>
    /// Site name
    /// </summary>
    public string SiteName
    {
        get
        {
            return ValidationHelper.GetString(ViewState["SiteName"], "");
        }
        set
        {
            ViewState["SiteName"] = value;
        }
    }


    protected void Page_Load(object sender, EventArgs e)
    {
        DataSet templates = PageTemplateInfoProvider.GetAllMasterTemplates();
        ucSelector.DataSource = templates;
        ucSelector.IDColumn = "PageTemplateID";
        ucSelector.DisplayNameColumn = "PageTemplateDisplayName";
        ucSelector.DescriptionColumn = "PageTemplateDescription";
        ucSelector.ObjectType = PortalObjectType.PAGETEMPLATE;
        ucSelector.DataBind();

        if (!RequestHelper.IsPostBack())
        {
            if (!DataHelper.DataSourceIsEmpty(templates))
            {
                int firstTemplateId = ValidationHelper.GetInteger(templates.Tables[0].Rows[0]["PageTemplateID"], 0);
                ucSelector.SelectedId = firstTemplateId;
            }
        }
    }


    /// <summary>
    /// Apply control settings
    /// </summary>
    public bool ApplySettings()
    {
        if (MasterTemplateId <= 0)
        {
            this.lblError.Text = ResHelper.GetString("TemplateSelection.SelectTemplate");
            return false;
        }
        else
        {
            // Update all culture versions
            TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
            DataSet ds = tree.SelectNodes(this.SiteName, "/", TreeProvider.ALL_CULTURES, false, "CMS.Root", null, null, -1, false);
            if (!DataHelper.DataSourceIsEmpty(ds))
            {
                foreach (DataRow dr in ds.Tables[0].Rows)
                {
                    // Update the document
                    TreeNode node = new TreeNode(dr, "CMS.Root", tree);
                    node.DocumentPageTemplateID = MasterTemplateId;
                    node.Update();

                    // Update search index for node
                    if ((node.PublishedVersionExists) && (SearchIndexInfoProvider.SearchEnabled))
                    {
                        SearchTaskInfoProvider.CreateTask(SearchTaskTypeEnum.Update, PredefinedObjectType.DOCUMENT, SearchHelper.ID_FIELD, node.GetSearchID());
                    }
                }
            }
        }

        return true;
    }


    protected override void OnPreRender(EventArgs e)
    {
        base.OnPreRender(e);

        this.lblError.Visible = (this.lblError.Text != "");
    }
}