using System;
using System.Data;
using System.Web.UI;
using System.Web.UI.WebControls;

using CMS.CMSHelper;
using CMS.GlobalHelper;
using CMS.Messaging;
using CMS.UIControls;

public partial class CMSModules_Messaging_Controls_Outbox : CMSUserControl, IPostBackEventHandler
{
    #region "Variables"

    protected string confirmationScript = null;
    protected string imagesUrl = null;

    private string mZeroRowsText = null;
    private int mPageSize = 10;
    private bool mShowOriginalMessage = true;
    private bool mMarkReadMessage = false;

    #endregion


    #region "Private properties"

    /// <summary>
    /// Sort direction
    /// </summary>
    protected string SortDirection
    {
        get
        {
            return ValidationHelper.GetString(ViewState["SortDirection"], "desc");
        }
        set
        {
            ViewState["SortDirection"] = value;
        }
    }


    /// <summary>
    /// Page index
    /// </summary>
    protected int PageIndex
    {
        get
        {
            return ValidationHelper.GetInteger(Session["PageIndex"], 0);
        }
        set
        {
            Session["PageIndex"] = value;
        }
    }


    /// <summary>
    /// Current message id
    /// </summary>
    protected int CurrentMessageId
    {
        get
        {
            return ValidationHelper.GetInteger(hdnSelectedId.Value, 0);
        }
        set
        {
            hdnSelectedId.Value = ValidationHelper.GetString(value, "");
        }
    }

    #endregion


    #region "Public properties"

    /// <summary>
    /// Zero rows text
    /// </summary>
    public string ZeroRowsText
    {
        get
        {
            return mZeroRowsText;
        }
        set
        {
            mZeroRowsText = value;
            gvMessages.EmptyDataText = value;
        }
    }


    /// <summary>
    /// Size of the page
    /// </summary>
    public int PageSize
    {
        get
        {
            return mPageSize;
        }
        set
        {
            mPageSize = value;
            gvMessages.PageSize = value;
        }
    }


    /// <summary>
    /// True if original message should be shown
    /// </summary>
    public bool ShowOriginalMessage
    {
        get
        {
            return mShowOriginalMessage;
        }
        set
        {
            mShowOriginalMessage = value;
        }
    }


    /// <summary>
    /// Mark read messages.
    /// </summary>
    public bool MarkReadMessage
    {
        get
        {
            return mMarkReadMessage;
        }
        set
        {
            mMarkReadMessage = value;
        }
    }

    #endregion


    protected override void OnPreRender(EventArgs e)
    {
        base.OnPreRender(e);

        // Register action script
        Page.ClientScript.RegisterClientScriptBlock(typeof(string), "mobGridHandling",
        ScriptHelper.GetScript(
            "function mobDeleteItem(selectedId) { \n" +
                "if(confirm(" + ScriptHelper.GetString(ResHelper.GetString("Messsaging.DeletionConfirmation")) + ")){" +
                    "selectedIdElem = document.getElementById('" + hdnSelectedId.ClientID + "'); \n" +
                    "if (selectedIdElem != null) { \n" +
                        "selectedIdElem.value=selectedId; \n" +
                        Page.ClientScript.GetPostBackEventReference(this, "DeleteItem", false) +
                        " }} return false;} \n" +

            "function mobViewItem(selectedId) { \n" +
                    "selectedIdElem = document.getElementById('" + hdnSelectedId.ClientID + "'); \n" +
                    "if (selectedIdElem != null) { \n" +
                        "selectedIdElem.value=selectedId; \n" +
                        Page.ClientScript.GetPostBackEventReference(this, "ViewItem", false) +
                        " } return false;} \n"
        ));

        // Set info message
        if (ucViewMessage.InformationText != "")
        {
            lblInfo.Visible = true;
            lblInfo.Text = ucViewMessage.InformationText;
        }

        // Set error message
        if (ucViewMessage.ErrorText != "")
        {
            lblError.Visible = true;
            lblError.Text = ucViewMessage.ErrorText;
        }
    }


    /// <summary>
    /// Page load.
    /// </summary>
    /// <param name="sender">Sender</param>
    /// <param name="e">Event arguments</param>
    protected void Page_Load(object sender, EventArgs e)
    {
        ucSendMessage.IsLiveSite = IsLiveSite;
        ucViewMessage.IsLiveSite = IsLiveSite;
        ucViewMessage.MessageMode = MessageModeEnum.Outbox;

        ReloadData();
    }


    /// <summary>
    /// Reload data
    /// </summary>
    public void ReloadData()
    {
        if (StopProcessing)
        {
            // Do nothing
            ucSendMessage.StopProcessing = true;
        }
        else
        {
            if (!RequestHelper.IsPostBack())
            {
                PageIndex = 0;
            }

            // Show content only for authenticated users
            if (CMSContext.CurrentUser.IsAuthenticated())
            {
                lblInfo.Visible = false;
                lblError.Visible = false;

                Visible = true;

                gvMessages.PageSize = PageSize;
                gvMessages.EmptyDataText = ZeroRowsText;

                lblNewMessageHeader.Text = ResHelper.GetString("Messaging.NewMessage");
                lblViewMessageHeader.Text = ResHelper.GetString("Messaging.ViewMessage");

                confirmationScript = "return confirm(" + ScriptHelper.GetString(ResHelper.GetString("Messsaging.DeletionConfirmation")) + ");";
                string confirmationScriptAll = "return confirm(" + ScriptHelper.GetString(ResHelper.GetString("Messsaging.AllDeletionConfirmation")) + ");";

                btnDelete.OnClientClick = confirmationScript;
                btnDeleteAll.OnClientClick = confirmationScriptAll;
                imagesUrl = UrlHelper.ImagesDirectory + "/CMSModules/Messaging/";
                ucViewMessage.MessageId = CurrentMessageId;

                // String initialization
                btnNewMessage.Text = ResHelper.GetString("Messaging.NewMessage");
                btnDeleteAll.Text = ResHelper.GetString("Messaging.DeleteAll");
                btnDelete.Text = ResHelper.GetString("Messaging.Delete");
                btnForward.Text = ResHelper.GetString("Messaging.Forward");
                btnBackToList.Text = ResHelper.GetString("Messaging.BackToList");

                gvMessages.RowDataBound += gvMessages_RowDataBound;
                gvMessages.PageIndexChanging += gvMessages_PageIndexChanging;
                gvMessages.Sorting += gvMessages_Sorting;
                ucSendMessage.SendButtonClick += ucSendMessage_SendButtonClick;

                btnNewMessage.Click += btnNewMessage_Click;
                btnDeleteAll.Click += btnDeleteAll_Click;
                btnForward.Click += btnForward_Click;
                btnDelete.Click += btnDelete_Click;
                btnBackToList.Click += btnBackToList_Click;

                // Bind the data
                Bind();
            }
            else
            {
                Visible = false;
            }
        }
    }


    void ucSendMessage_SendButtonClick(object sender, EventArgs e)
    {
        // If no error, inform user
        if (ucSendMessage.ErrorText == string.Empty)
        {
            lblInfo.Visible = true;
            lblInfo.Text = ucSendMessage.InformationText;
            btnBackToList_Click(sender, e);
            Bind();
        }
    }


    void gvMessages_PageIndexChanging(object sender, GridViewPageEventArgs e)
    {
        PageIndex = e.NewPageIndex;
        Bind();
    }

    #region "Message actions"

    /// <summary>
    /// New message
    /// </summary>
    private void NewMessage()
    {
        // Initialize new message control
        pnlBackToList.Visible = true;
        pnlList.Visible = false;
        pnlNew.Visible = true;
        pnlActions.Visible = false;

        // Initilaize new and view message
        ucSendMessage.SendMessageMode = CMSModules_Messaging_Controls_SendMessage.SendMessageEnum.NewMessage;
        ucSendMessage.MessageId = 0;
        ucSendMessage.ReloadData();
    }


    /// <summary>
    /// View message
    /// </summary>
    private void ViewMessage()
    {
        pnlActions.Visible = true;
        pnlList.Visible = false;
        pnlNew.Visible = false;
        pnlView.Visible = true;
        pnlBackToList.Visible = true;

        // Initialize view message control
        ucViewMessage.MessageId = CurrentMessageId;
        ucViewMessage.ReloadData();
    }


    /// <summary>
    /// Forward message
    /// </summary>
    private void ForwardMesssage()
    {
        pnlBackToList.Visible = true;
        pnlList.Visible = false;
        pnlNew.Visible = true;
        pnlView.Visible = ShowOriginalMessage;
        pnlActions.Visible = false;

        // Initilaize new and view message
        ucSendMessage.MessageId = CurrentMessageId;
        ucSendMessage.MessageSubject = ResHelper.GetString("Messaging.ForwardSign");
        ucSendMessage.SendMessageMode = CMSModules_Messaging_Controls_SendMessage.SendMessageEnum.ForwardMessage;
        ucSendMessage.ReloadData();
        ucViewMessage.MessageId = CurrentMessageId;
        ucViewMessage.ReloadData();
    }


    /// <summary>
    /// Delete message
    /// </summary>
    private void DeleteMessage()
    {
        try
        {
            MessageInfoProvider.DeleteSentMessage(CurrentMessageId);
            lblInfo.Visible = true;
            lblInfo.Text = ResHelper.GetString("Messsaging.MessageDeleted");
            pnlList.Visible = true;
            pnlNew.Visible = false;
            pnlView.Visible = false;

            // Reload data
            Bind();
        }
        catch (Exception ex)
        {
            lblError.Visible = true;
            lblError.Text = ex.Message;
        }
    }

    #endregion


    #region "Buttons actions"

    void btnDeleteAll_Click(object sender, EventArgs e)
    {
        try
        {
            MessageInfoProvider.DeleteSentMessages(CMSContext.CurrentUser.UserID);
            lblInfo.Visible = true;
            lblInfo.Text = ResHelper.GetString("Messsaging.AllMessagesDeleted");
            pnlList.Visible = true;
            pnlNew.Visible = false;
            pnlView.Visible = false;

            // Reload data
            Bind();
        }
        catch (Exception ex)
        {
            lblError.Visible = true;
            lblError.Text = ex.Message;
        }
    }


    void btnBackToList_Click(object sender, EventArgs e)
    {
        pnlList.Visible = true;
        pnlNew.Visible = false;
        pnlView.Visible = false;
        pnlBackToList.Visible = false;
    }


    void btnDelete_Click(object sender, EventArgs e)
    {
        DeleteMessage();
    }


    void btnForward_Click(object sender, EventArgs e)
    {
        ForwardMesssage();
    }


    void btnNewMessage_Click(object sender, EventArgs e)
    {
        NewMessage();
    }

    #endregion


    #region "Grid methods"

    /// <summary>
    /// Bind the data
    /// </summary>
    private void Bind()
    {
        gvMessages.Columns[1].HeaderText = ResHelper.GetString("Messaging.To");
        gvMessages.Columns[1].SortExpression = "MessageRecipientUserID";
        gvMessages.Columns[2].HeaderText = ResHelper.GetString("general.subject");
        gvMessages.Columns[2].SortExpression = "MessageSubject";
        gvMessages.Columns[3].HeaderText = ResHelper.GetString("Messaging.Date");
        gvMessages.Columns[3].SortExpression = "MessageSent";
        if (MarkReadMessage)
        {
            gvMessages.Columns[4].HeaderText = ResHelper.GetString("Messaging.DateRead");
            gvMessages.Columns[4].SortExpression = "MessageRead";
        }
        else
        {
            gvMessages.Columns[4].Visible = false;
        }
        const string columns = "MessageID,MessageRecipientNickName,MessageSent,MessageSubject,MessageRead,MessageRecipientDeleted,MessageLastModified,MessageRecipientUserID";
        gvMessages.DataSource = MessageInfoProvider.GetSentMessages(CMSContext.CurrentUser.UserID, "MessageSent " + SortDirection, -1, columns);
        gvMessages.PageIndex = PageIndex;
        gvMessages.DataBind();

        lblFooter.Text = string.Format(ResHelper.GetString("Messaging.NumOfMessages"), MessageInfoProvider.GetSentMessagesCount(CMSContext.CurrentUser.UserID));
    }


    void gvMessages_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        DataRowView rowView = ((DataRowView)e.Row.DataItem);
        if (e.Row.RowType == DataControlRowType.DataRow)
        {
            string cssClass = ((e.Row.RowIndex % 2) == 0) ? "OddRow" : "EvenRow";

            // Mark read message
            if (MarkReadMessage)
            {
                DateTime messageRead = ValidationHelper.GetDateTime(DataHelper.GetDataRowValue(rowView.Row, "MessageRead"), DateTimeHelper.ZERO_TIME);
                if (messageRead != DateTimeHelper.ZERO_TIME)
                {
                    cssClass += "Read";
                }
            }
            e.Row.CssClass = cssClass;
        }
    }


    void gvMessages_Sorting(object sender, GridViewSortEventArgs e)
    {
        SortDirection = (SortDirection == "asc") ? "desc" : "asc";
        string sortExp = e.SortExpression + " " + SortDirection;

        DataSet ds = (DataSet)gvMessages.DataSource;

        if (!DataHelper.DataSourceIsEmpty(ds))
        {
            DataView view = ds.Tables[0].DefaultView;
            view.Sort = sortExp;
            gvMessages.DataSource = view;
            gvMessages.DataBind();
        }
    }

    #endregion

    protected string GetDateRead(object messageReadDate)
    {
        DateTime messageRead = ValidationHelper.GetDateTime(messageReadDate, DateTimeHelper.ZERO_TIME);
        if (messageRead != DateTimeHelper.ZERO_TIME)
        {
            return CMSContext.ConvertDateTime(messageRead, this).ToString();
        }
        return ResHelper.GetString("Messaging.OutboxMessageUnread");
    }


    #region "IPostBackEventHandler Members"

    public void RaisePostBackEvent(string eventArgument)
    {
        if ((eventArgument == null))
        {
            return;
        }

        int selectedId = ValidationHelper.GetInteger(hdnSelectedId.Value, 0);


        if (selectedId != 0)
        {
            switch (eventArgument.ToLower())
            {
                case "deleteitem":
                    // Delete message
                    DeleteMessage();
                    break;

                case "viewitem":
                    // View message
                    ViewMessage();
                    break;
            }
        }
    }

    #endregion
}
