using System;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Data;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using System.Xml;

using CMS.GlobalHelper;
using CMS.SettingsProvider;
using CMS.CMSHelper;
using CMS.Staging;
using CMS.PortalEngine;
using CMS.TreeEngine;
using CMS.FileManager;
using CMS.WorkflowEngine;
using CMS.SiteProvider;

public partial class CMSModules_Staging_Tools_Tasks_View : CMSStagingPage
{
    protected void Page_Load(object sender, EventArgs e)
    {
        // check 'Manage servers' permission
        if (!CMSContext.CurrentUser.IsAuthorizedPerResource("cms.staging", "ManageTasks"))
        {
            RedirectToAccessDenied("cms.staging", "ManageTasks");
        }

        CurrentMaster.Title.TitleText = ResHelper.GetString("Task.ViewHeader");
        CurrentMaster.Title.TitleImage = UrlHelper.ImagesDirectory + "/CMSModules/Staging/Tasks/task.gif";

        int taskId = ValidationHelper.GetInteger(Request.QueryString["taskid"], 0);
        TaskInfo ti = TaskInfoProvider.GetTaskInfo(taskId);
        if (ti != null)
        {
            DataSet ds = GetDataSet(ti.TaskData, TaskHelper.GetTaskTypeString(ti.TaskType), ti.TaskObjectType);
            if (!DataHelper.DataSourceIsEmpty(ds))
            {
                // Prepare list of tables
                SortedDictionary<string, DataTable> tables = new SortedDictionary<string, DataTable>();
                foreach (DataTable dt in ds.Tables)
                {
                    if (!DataHelper.DataSourceIsEmpty(dt))
                    {
                        tables.Add(ResHelper.GetString("ObjectType." + dt.TableName), dt);
                    }
                }

                // Generate the tables
                foreach (DataTable dt in tables.Values)
                {
                    pnlContent.Controls.Add(new LiteralControl("<h3>" + ResHelper.GetString("ObjectType." + dt.TableName) + "</h3>"));

                    if ((dt.Columns.Count >= 6) && !dt.TableName.Equals("ObjectTranslation", StringComparison.InvariantCultureIgnoreCase))
                    {
                        // Write all rows
                        foreach (DataRow dr in dt.Rows)
                        {
                            StringBuilder sb = new StringBuilder();

                            sb.Append("<table cellspacing=\"0\" cellpadding=\"3\" rules=\"rows\" border=\"1\" style=\"border-collapse:collapse;\" width=\"100%\">");

                            // Add values
                            foreach (DataColumn dc in dt.Columns)
                            {
                                sb.Append("<tr><td>");
                                sb.Append("<strong>" + dc.ColumnName + "</strong>");
                                sb.Append("</td><td style=\"width: 100%;\">");

                                string content = null;

                                // Binary columns
                                if ((dc.DataType == typeof(byte[])) && (dr[dc.ColumnName] != DBNull.Value))
                                {
                                    byte[] data = (byte[])dr[dc.ColumnName];
                                    content = "<" + ResHelper.GetString("General.BinaryData") + ", " + DataHelper.GetSizeString(data.Length) + ">";
                                }
                                else
                                {
                                    content = ValidationHelper.GetString(dr[dc.ColumnName], "");
                                }

                                bool standard = true;
                                switch (dc.ColumnName.ToLower())
                                {
                                    // Document content
                                    case "documentcontent":
                                        EditableItems items = new EditableItems();
                                        items.LoadContentXml(ValidationHelper.GetString(dr[dc.ColumnName], ""));

                                        // Add regions
                                        foreach (DictionaryEntry region in items.EditableRegions)
                                        {
                                            sb.Append("<span class=\"VersionEditableRegionTitle\">" + (string)region.Key + "</span>");

                                            string regionContent = HTMLHelper.ResolveUrls((string)region.Value, Request.ApplicationPath);

                                            sb.Append("<span class=\"VersionEditableRegionText\">" + regionContent + "</span>");
                                        }

                                        // Add web parts
                                        foreach (DictionaryEntry part in items.EditableWebParts)
                                        {
                                            sb.Append("<span class=\"VersionEditableWebPartTitle\">" + (string)part.Key + "</span>");

                                            string regionContent = HTMLHelper.ResolveUrls((string)part.Value, Request.ApplicationPath);
                                            sb.Append("<span class=\"VersionEditableWebPartText\">" + regionContent + "</span>");
                                        }

                                        standard = false;
                                        break;

                                    // XML columns
                                    case "pagetemplatewebparts":
                                    case "webpartproperties":
                                    case "reportparameters":
                                    case "classformdefinition":
                                    case "classxmlschema":
                                    case "classformlayout":
                                        content = HTMLHelper.ReformatHTML(content);
                                        break;

                                    // File columns
                                    case "metafilename":
                                        {
                                            Guid metaFileGuid = ValidationHelper.GetGuid(dr["MetaFileGuid"], Guid.Empty);
                                            if (metaFileGuid != Guid.Empty)
                                            {
                                                string metaFileName = ValidationHelper.GetString(dr["MetaFileName"], "");

                                                content = "<a href=\"" + ResolveUrl(MetaFileInfoProvider.GetMetaFileUrl(metaFileGuid, metaFileName)) + "\" target=\"_blank\" >" + HTMLHelper.HTMLEncode(metaFileName) + "</a>";
                                                sb.Append(content);

                                                standard = false;
                                            }
                                        }
                                        break;
                                }

                                // Standard rendering
                                if (standard)
                                {
                                    content = HTMLHelper.HTMLEncode(content);
                                    content = TextHelper.EnsureLineEndings(content, "<br />");
                                    content = content.Replace("\t", "&nbsp;&nbsp;&nbsp;&nbsp;");
                                    sb.Append("<div style=\"max-height: 300px; overflow: auto;\">" + content + "</div>");
                                }

                                sb.Append("</td></tr>");
                            }

                            sb.Append("</table><br />\n");

                            pnlContent.Controls.Add(new LiteralControl(sb.ToString()));
                        }
                    }
                    else
                    {
                        GridView newGrid = new GridView();
                        newGrid.ID = "grid" + dt.TableName;
                        newGrid.EnableViewState = false;
                        newGrid.HeaderStyle.HorizontalAlign = CultureHelper.IsUICultureRTL() ? HorizontalAlign.Right : HorizontalAlign.Left;
                        newGrid.CellPadding = 3;
                        newGrid.GridLines = GridLines.Horizontal;

                        pnlContent.Controls.Add(newGrid);

                        newGrid.DataSource = ds;
                        newGrid.DataMember = dt.TableName;

                        newGrid.DataBind();
                    }
                }
            }
        }
    }


    /// <summary>
    /// Returns the dataset loaded from the given document data
    /// </summary>
    /// <param name="documentData">Document data to make the dataset from</param>
    /// <param name="taskType">Task type</param>
    /// <param name="taskObjectType">Task object type</param>
    protected virtual DataSet GetDataSet(string documentData, string taskType, string taskObjectType)
    {
        DataSet ds = null;

        // Process the task
        switch (taskType.ToLower())
        {
            // Document tasks
            case "updatedoc":
            case "deletedoc":
            case "deleteallcultures":
            case "publishdoc":
            case "archivedoc":
            case "movedoc":
                {
                    ds = new DataSet();

                    switch (taskType.ToLower())
                    {
                        case "updatedoc":
                            {
                                // Attachment table
                                AttachmentInfo ai = new AttachmentInfo(null);
                                DataSet attachmentsDS = CMSObjectHelper.GetObjectsDataSet(OperationTypeEnum.Staging, ai, false);
                                DataHelper.TransferTables(ds, attachmentsDS);

                                // Page template tables
                                PageTemplateInfo pti = new PageTemplateInfo();
                                DataSet templatesDS = CMSObjectHelper.GetObjectsDataSet(OperationTypeEnum.Staging, pti, true);
                                DataHelper.TransferTables(ds, templatesDS);
                            }
                            break;

                        case "publishdoc":
                            {
                                // Attachment history table
                                AttachmentHistoryInfo ai = new AttachmentHistoryInfo(null);
                                DataSet attachmentsDS = CMSObjectHelper.GetObjectsDataSet(OperationTypeEnum.Staging, ai, false);
                                DataHelper.TransferTables(ds, attachmentsDS);
                            }
                            break;
                    }

                    // Translation table
                    ds.Tables.Add(TranslationHelper.GetEmptyTable());
                }
                break;

            // Update object
            case "updateobj":
                {
                    // Get the object definition
                    IInfoObject infoObj = GetObject(null, taskObjectType);

                    // Get the DataSet
                    ds = CMSObjectHelper.GetObjectsDataSet(OperationTypeEnum.Staging, infoObj, true);
                    if (ds != null)
                    {
                        ds.Tables.Add(TranslationHelper.GetEmptyTable());
                    }
                }
                break;

            // Delete object
            case "deleteobj":
                {
                    // Get the object definition
                    IInfoObject infoObj = GetObject(null, taskObjectType);

                    // Get the DataSet
                    ds = CMSObjectHelper.GetObjectsDataSet(OperationTypeEnum.Staging, infoObj, false);
                    if (ds != null)
                    {
                        ds.Tables.Add(TranslationHelper.GetEmptyTable());
                    }
                }
                break;

            // Unknown task
            default:
                ds = new DataSet();
                break;
        }

        XmlParserContext xmlContext = new XmlParserContext(null, null, null, XmlSpace.None);
        XmlReader reader = new XmlTextReader(documentData, XmlNodeType.Element, xmlContext);

        ds = DataHelper.ReadDataSetFromXml(ds, reader, null, null);
        //ds.ReadXml(reader);

        return ds;
    }


    /// <summary>
    /// Gets the object created from the given DataRow
    /// </summary>
    /// <param name="objectRow">Object DataRow</param>
    /// <param name="objectType">Object type</param>
    protected IInfoObject GetObject(DataRow objectRow, string objectType)
    {
        IInfoObject infoObj = CMSObjectHelper.GetObject(objectRow, objectType);
        if (infoObj == null)
        {
            throw new Exception("Object type '" + objectType + "' not found.");
        }

        return infoObj;
    }
}