using System;
using System.Data;
using System.Web.UI;
using System.Web.UI.WebControls;

using CMS.DataEngine;
using CMS.GlobalHelper;
using CMS.CMSHelper;
using CMS.SiteProvider;
using CMS.UIControls;

public partial class SiteManager_Administration_Permissions_Permissions_Matrix : AdministrationPage, IPostBackEventHandler
{
    #region "Protected variables"

    protected int id = 0;
    protected string type = string.Empty;

    #endregion


    #region "Page events"

    protected override void OnPreInit(EventArgs e)
    {
        ((Panel)this.CurrentMaster.PanelBody.FindControl("pnlContent")).CssClass = "";
        base.OnPreInit(e);
    }


    protected void Page_Load(object sender, EventArgs e)
    {
        // Check "read" permission
        if (!CMSContext.CurrentUser.IsAuthorizedPerResource("CMS.Permissions", "Read"))
        {
            RedirectToAccessDenied("CMS.Permissions", "Read");
        }

        // Get the settings
        id = QueryHelper.GetInteger("id", 0);
        if (id > 0)
        {
            type = QueryHelper.GetString("type", "");
        }
    }


    protected override void OnPreRender(EventArgs e)
    {
        if (id > 0)
        {
            // Render permission matrix
            CreateMatrix();
        }

        base.OnPreRender(e);
    }

    #endregion


    #region "Private methods"

    /// <summary>
    /// Returns HTML code of the permission matrix for specified class or resource.
    /// </summary>
    private void CreateMatrix()
    {
        // Retrive permission matrix data

        string imagesUrl = ResolveUrl(UrlHelper.ImagesDirectory + "/CMSSiteManager/Administration/Permissions/");

        DataSet permissionMatrixSet = PermissionNameInfoProvider.GetPermissionMatrix(id, type, GetSiteID(Request.QueryString["siteid"]), true);

        // Render the matrix
        if (permissionMatrixSet.Tables[0].Rows.Count == 0)
        {
            lblInfo.Text = ResHelper.GetString("Administration-Permissions_Matrix.EmptyMatrix");
        }
        else
        {
            int lastRoleID = Convert.ToInt32(permissionMatrixSet.Tables[0].Rows[0]["RoleID"]);

            // Render header
            TableRow headerRow = new TableRow();

            // Blank cell for the role names
            TableCell newCell = new TableCell();
            headerRow.Cells.Add(newCell);

            foreach (DataRow dr in permissionMatrixSet.Tables[0].Rows)
            {
                if (lastRoleID != Convert.ToInt32(dr["RoleID"]))
                {
                    break;
                }
                else
                {
                    newCell = new TableCell();

                    newCell.CssClass = "MatrixHeader";
                    newCell.Text = HTMLHelper.HTMLEncode(Convert.ToString(dr["PermissionDisplayName"]));
                    newCell.ToolTip = HTMLHelper.HTMLEncode(Convert.ToString(dr["PermissionDescription"]));
                    newCell.HorizontalAlign = HorizontalAlign.Center;
                    newCell.Width = new Unit(75, UnitType.Pixel);

                    headerRow.Cells.Add(newCell);
                }
            }

            newCell = new TableCell();
            //newCell.Width = new Unit(100, UnitType.Percentage);
            headerRow.Cells.Add(newCell);
            tblMatrix.Rows.Add(headerRow);

            // Render matrix
            lastRoleID = -1;
            TableRow newRow = null;
            // Render matrix rows
            foreach (DataRow dr in permissionMatrixSet.Tables[0].Rows)
            {
                if (lastRoleID != Convert.ToInt32(dr["RoleID"]))
                {
                    if (newRow != null)
                    {
                        newCell = new TableCell();
                        newRow.Cells.Add(newCell);
                        tblMatrix.Rows.Add(newRow);
                    }
                    // New row - start with role name
                    newRow = new TableRow();
                    newCell = new TableCell();
                    newCell.CssClass = "MatrixHeader";
                    newCell.Text = HTMLHelper.HTMLEncode(Convert.ToString(dr["RoleDisplayName"]));
                    newCell.Wrap = false;
                    newCell.Width = new Unit(150, UnitType.Pixel);
                    newRow.Cells.Add(newCell);
                    lastRoleID = Convert.ToInt32(dr["RoleID"]);
                }

                newCell = new TableCell();
                newCell.CssClass = "MatrixCell";

                if (Convert.ToInt32(dr["Allowed"]) == 1)
                {
                    newCell.Text = "<img src=\"" + imagesUrl + "allowed.gif\" style=\"cursor:pointer;\" title=\"" + HTMLHelper.HTMLEncode(dr["PermissionDescription"].ToString()) + "\" onclick=\"" + Page.ClientScript.GetPostBackEventReference(this, Convert.ToString(dr["RoleID"]) + ";" + Convert.ToString(dr["PermissionID"]) + ";0") + "\">";
                }
                else
                {
                    newCell.Text = "<img src=\"" + imagesUrl + "denied.gif\" style=\"cursor:pointer;\" title=\"" + HTMLHelper.HTMLEncode(dr["PermissionDescription"].ToString()) + "\" onclick=\"" + Page.ClientScript.GetPostBackEventReference(this, Convert.ToString(dr["RoleID"]) + ";" + Convert.ToString(dr["PermissionID"]) + ";1") + "\">";
                }

                if (newRow != null)
                {
                    newRow.Cells.Add(newCell);
                }
            }
            // Add the latest row if present
            if (newRow != null)
            {
                newRow.Cells.Add(new TableCell());
                tblMatrix.Rows.Add(newRow);
            }
        }
    }

    #endregion


    #region "Postback handling"

    public void RaisePostBackEvent(string eventArgument)
    {
        // Check "Manage" permission
        if (!CMSContext.CurrentUser.IsAuthorizedPerResource("CMS.Permissions", "Manage"))
        {
            RedirectToAccessDenied("CMS.Permissions", "Manage");
        }

        if (eventArgument.Split(';').Length == 3)
        {
            int roleId = Convert.ToInt32(eventArgument.Split(';')[0]);
            int permissionId = Convert.ToInt32(eventArgument.Split(';')[1]);
            string value = eventArgument.Split(';')[2];

            RolePermissionInfo rpi = new RolePermissionInfo();
            rpi.RoleID = roleId;
            rpi.PermissionID = permissionId;

            // Set permissions
            if (value == "1")
            {
                RolePermissionInfoProvider.SetRolePermissionInfo(rpi);
            }
            else if (value == "0")
            {
                RolePermissionInfoProvider.DeleteRolePermissionInfo(rpi);
            }
        }
    }

    #endregion
}
