using System;
using System.Data;
using System.Web;
using System.Web.UI.WebControls;

using CMS.PortalControls;
using CMS.GlobalHelper;
using CMS.Controls;

public partial class CMSWebParts_WebServices_RepeaterForWebService : CMSAbstractWebPart
{
    #region "Variables"

    // Web service URL
    protected string wsURL = "";

    // Web service parameters
    protected string wsParameters = "";

    // Transformation name
    protected string transformationName = "";

    #endregion


    #region "Public properties"

    /// <summary>
    /// Gets or sets the value that indicates whether to hide control when no data found
    /// </summary>
    public bool HideControlForZeroRows
    {
        get
        {
            return ValidationHelper.GetBoolean(GetValue("HideControlForZeroRows"), basicRepeater.HideControlForZeroRows);
        }
        set
        {
            SetValue("HideControlForZeroRows", value);
            basicRepeater.HideControlForZeroRows = value;
        }
    }


    /// <summary>
    /// Gets or sets the text which is displayed when no data found
    /// </summary>
    public string ZeroRowsText
    {
        get
        {
            return DataHelper.GetNotEmpty(ValidationHelper.GetString(GetValue("ZeroRowsText"), basicRepeater.ZeroRowsText), basicRepeater.ZeroRowsText);
        }
        set
        {
            SetValue("ZeroRowsText", value);
            basicRepeater.ZeroRowsText = value;
        }
    }
    #endregion


    #region "Data binding properties"

    /// <summary>
    /// Gets or sets the value that indicates whether to databind by default
    /// </summary>
    public bool DataBindByDefault
    {
        get
        {
            return ValidationHelper.GetBoolean(GetValue("DataBindByDefault"), basicRepeater.DataBindByDefault);
        }
        set
        {
            SetValue("DataBindByDefault", value);
            basicRepeater.DataBindByDefault = value;
        }
    }

    #endregion


    #region "Webservices properties"

    /// <summary>
    /// Gets or sets the parameters of the web service
    /// </summary>
    public string WebServiceParameters
    {
        get
        {
            return ValidationHelper.GetString(GetValue("WebServiceParameters"), wsParameters).Replace("\"", "'");
        }
        set
        {
            SetValue("WebServiceParameters", value);
            wsParameters = value;
        }
    }


    /// <summary>
    /// Gets or sets the url of the web service
    /// </summary>
    public string WebServiceURL
    {
        get
        {
            return ValidationHelper.GetString(GetValue("WebServiceURL"), wsURL);
        }
        set
        {
            SetValue("WebServiceURL", value);
            wsURL = value;
        }
    }


    /// <summary>
    /// Gets or sets the name of the transforamtion which is used for displaying the results
    /// </summary>
    public string TransformationName
    {
        get
        {
            return ValidationHelper.GetString(GetValue("TransformationName"), transformationName);
        }
        set
        {
            SetValue("TransformationName", value);
            transformationName = value;
        }
    }

    #endregion


    /// <summary>
    /// Content loaded event handler
    /// </summary>
    public override void OnContentLoaded()
    {
        base.OnContentLoaded();
        SetupControl();
    }


    /// <summary>
    /// Initializes the control properties
    /// </summary>
    protected void SetupControl()
    {
        if (StopProcessing)
        {
            basicRepeater.Visible = false;
        }
        else
        {
            // Setup the control
            // Public
            basicRepeater.HideControlForZeroRows = HideControlForZeroRows;
            basicRepeater.ZeroRowsText = ZeroRowsText;
            basicRepeater.DataBindByDefault = DataBindByDefault;

            // Get web service URL, parameters and transformation name
            wsURL = WebServiceURL;
            wsParameters = WebServiceParameters;
            transformationName = TransformationName;

            if (wsURL != "")
            {
                // Get data from web service
                DataLoad();
            }
        }
    }


    /// <summary>
    /// Load data from web service to Repeater.
    /// </summary>
    protected void DataLoad()
    {
        // Web service object
        WS.WebService webService = new WS.WebService();
        // Set web service url
        if (!wsURL.Contains("http://"))
        {
            webService.Url = "http://" + wsURL;
        }
        else
        {
            webService.Url = wsURL;
        }

        try
        {
            // Get dataset from web service
            DataSet ds = webService.GetDataSet(wsParameters);
            if (!DataHelper.DataSourceIsEmpty(ds))
            {
                basicRepeater.DataSource = ds;
                if ((transformationName != null) && (transformationName.Trim() != ""))
                {
                    // Get url to transformation and load it
                    basicRepeater.ItemTemplate = CMSDataProperties.LoadTransformation(basicRepeater, transformationName, false);
                }
            }
            else
            {
                if (HideControlForZeroRows)
                {
                    Visible = false;
                }
            }
        }
        catch (Exception ex)
        {
            basicRepeater.Visible = false;

            Label lblError = new Label();
            lblError.Text = "Error occurred: " + ex.Message;

            Controls.Add(lblError);
        }
    }


    /// <summary>
    /// Reload control's data.
    /// </summary>
    public override void ReloadData()
    {
        base.ReloadData();
        SetupControl();
        basicRepeater.ReloadData(true);
    }
}
